@@ -55,8 +55,8 @@ class ConvertOptionParser(optrecurse.RecursiveOptionParser, object):
     def add_duplicates_option(self, default="msgctxt"):
         """adds an option to say what to do with duplicate strings"""
         self.add_option("", "--duplicates", dest="duplicatestyle", default=default,
-            type="choice", choices=["msgid_comment", "msgctxt", "merge", "keep", "msgid_comment_all"],
-            help="what to do with duplicate strings (identical source text): merge, msgid_comment, msgctxt, keep, msgid_comment_all (default: '%s')" % default, metavar="DUPLICATESTYLE")
+            type="choice", choices=["msgctxt", "merge"],
+            help="what to do with duplicate strings (identical source text): merge, msgctxt (default: '%s')" % default, metavar="DUPLICATESTYLE")
         self.passthrough.append("duplicatestyle")
 
     def add_multifile_option(self, default="single"):
