@@ -27,7 +27,6 @@ usage instructions
 
 from translate.convert import dtd2po
 from translate.convert import prop2po
-from translate.convert import html2po
 from translate.convert import mozfunny2prop
 from translate.storage import xpi
 from translate.convert import convert
@@ -37,7 +36,7 @@ def main(argv=None):
             ("*", "*"): ("*", convert.copyinput),
             "*": ("*", convert.copyinput)}
     # handle formats that convert to .po files
-    converters = [("dtd", dtd2po.convertdtd), ("properties", prop2po.convertprop), ("xhtml", html2po.converthtml), ("html", html2po.converthtml),
+    converters = [("dtd", dtd2po.convertdtd), ("properties", prop2po.convertmozillaprop),
             ("it", mozfunny2prop.it2po), ("ini", mozfunny2prop.ini2po), ("inc", mozfunny2prop.inc2po)]
     for format, converter in converters:
         formats[(format, format)] = (format + ".po", converter)
