@@ -26,15 +26,6 @@ from translate.misc import quote
 from translate.convert import prop2po
 from translate.misc.wStringIO import StringIO
 
-def encodepropline(line):
-    """helper which strips off any end of line, encodes for properties file, and adds on the end of line"""
-    strippedline = line.rstrip("\n")
-    if line == strippedline:
-        ending = ""
-    else:
-        ending = line[len(strippedline)-len(line):]
-    return quote.mozillapropertiesencode(strippedline) + ending
-
 def inc2prop(lines):
     """convert a .inc file with #defines in it to a properties file"""
     yield "# converted from #defines file\n"
@@ -86,36 +77,36 @@ def funny2prop(lines, itencoding="cp1252"):
     hashstarts = len([line for line in lines if line.startswith("#")])
     if hashstarts:
         for line in inc2prop(lines):
-            yield encodepropline(line)
+            yield line
     else:
         for line in it2prop(lines, encoding=itencoding):
-            yield encodepropline(line)
+            yield line
 
 def inc2po(inputfile, outputfile, templatefile, encoding=None, pot=False, duplicatestyle="msgctxt"):
     """wraps prop2po but converts input/template files to properties first"""
     inputlines = inputfile.readlines()
-    inputproplines = [encodepropline(line) for line in inc2prop(inputlines)]
+    inputproplines = [line for line in inc2prop(inputlines)]
     inputpropfile = StringIO("".join(inputproplines))
     if templatefile is not None:
         templatelines = templatefile.readlines()
-        templateproplines = [encodepropline(line) for line in inc2prop(templatelines)]
+        templateproplines = [line for line in inc2prop(templatelines)]
         templatepropfile = StringIO("".join(templateproplines))
     else:
         templatepropfile = None
-    return prop2po.convertprop(inputpropfile, outputfile, templatepropfile, pot=pot, duplicatestyle=duplicatestyle)
+    return prop2po.convertprop(inputpropfile, outputfile, templatepropfile, personality="mozilla", pot=pot, duplicatestyle=duplicatestyle)
 
 def it2po(inputfile, outputfile, templatefile, encoding="cp1252", pot=False, duplicatestyle="msgctxt"):
     """wraps prop2po but converts input/template files to properties first"""
     inputlines = inputfile.readlines()
-    inputproplines = [encodepropline(line) for line in it2prop(inputlines, encoding=encoding)]
+    inputproplines = [line for line in it2prop(inputlines, encoding=encoding)]
     inputpropfile = StringIO("".join(inputproplines))
     if templatefile is not None:
         templatelines = templatefile.readlines()
-        templateproplines = [encodepropline(line) for line in it2prop(templatelines, encoding=encoding)]
+        templateproplines = [line for line in it2prop(templatelines, encoding=encoding)]
         templatepropfile = StringIO("".join(templateproplines))
     else:
         templatepropfile = None
-    return prop2po.convertprop(inputpropfile, outputfile, templatepropfile, pot=pot, duplicatestyle=duplicatestyle)
+    return prop2po.convertprop(inputpropfile, outputfile, templatepropfile, personality="mozilla", pot=pot, duplicatestyle=duplicatestyle)
 
 def ini2po(inputfile, outputfile, templatefile, encoding="UTF-8", pot=False, duplicatestyle="msgctxt"):
     return it2po(inputfile=inputfile, outputfile=outputfile, templatefile=templatefile, encoding=encoding, pot=pot, duplicatestyle=duplicatestyle)
