@@ -70,6 +70,10 @@ def convertodf(inputfile, outputfile, templates, engine):
     @contextmanager
     def store_context():
         store = factory.getobject(outputfile)
+        try:
+            store.setfilename(store.getfilenode('NoName'), inputfile.name)
+        except:
+            print "couldn't set origin filename"
         yield store
         store.save()
 
@@ -78,7 +82,12 @@ def convertodf(inputfile, outputfile, templates, engine):
             translate_toolkit_implementation(store)
         else:
             itools_implementation(store)
-    
+
+    # Since the convertoptionsparser will give us an open file, we risk that
+    # it could have been opened in non-binary mode on Windows, and then we'll
+    # have problems, so let's make sure we have what we want.
+    inputfile.close()
+    inputfile = file(inputfile.name, mode='rb')
     with_(store_context(), with_block)
     return True
 
