@@ -58,6 +58,7 @@ class rephp:
                     self.inputdict[location] = inputstring
 
     def convertline(self, line):
+        line = unicode(line, 'utf-8')
         returnline = ""
         # handle multiline msgid if we're in one
         if self.inmultilinemsgid:
@@ -75,21 +76,29 @@ class rephp:
         else:
             line = quote.rstripeol(line)
             equalspos = line.find('=')
+            hashpos = line.find("#")
             # if no equals, just repeat it
             if equalspos == -1:
                 returnline = quote.rstripeol(line)+eol
+            elif 0 <= hashpos < equalspos:
+                # Assume that this is a '#' comment line
+                returnline = quote.rstripeol(line)+eol
             # otherwise, this is a definition
             else:
                 # now deal with the current string...
                 key = line[:equalspos].strip()
                 lookupkey = key.replace(" ", "")
                 # Calculate space around the equal sign
-                prespace = line.lstrip()[line.lstrip().find(']')+1:equalspos]
+                prespace = line[len(line[:equalspos].rstrip()):equalspos]
                 postspacestart = len(line[equalspos+1:])
                 postspaceend = len(line[equalspos+1:].lstrip())
                 postspace = line[equalspos+1:equalspos+(postspacestart-postspaceend)+1]
                 self.quotechar = line[equalspos+(postspacestart-postspaceend)+1]
-                inlinecomment = line[line.rfind("%s;" % self.quotechar)+2:]
+                inlinecomment_pos = line.rfind("%s;" % self.quotechar)
+                if inlinecomment_pos > -1:
+                    inlinecomment = line[inlinecomment_pos+2:]
+                else:
+                    inlinecomment = ""
                 if self.inputdict.has_key(lookupkey):
                     self.inecho = False
                     value = php.phpencode(self.inputdict[lookupkey], self.quotechar)
