@@ -31,10 +31,14 @@ from translate.storage import properties
 
 class prop2po:
     """convert a .properties file to a .po file for handling the translation..."""
-    def convertstore(self, thepropfile, duplicatestyle="msgctxt"):
+    def convertstore(self, thepropfile, personality="java", duplicatestyle="msgctxt"):
         """converts a .properties file to a .po file..."""
+        self.personality = personality
         thetargetfile = po.pofile()
-        targetheader = thetargetfile.makeheader(charset="UTF-8", encoding="8bit", x_accelerator_marker="&")
+        if self.personality == "mozilla" or self.personality == "skype":
+            targetheader = thetargetfile.makeheader(charset="UTF-8", encoding="8bit", x_accelerator_marker="&")
+        else:
+            targetheader = thetargetfile.makeheader(charset="UTF-8", encoding="8bit")
         targetheader.addnote("extracted from %s" % thepropfile.filename, "developer")
         # we try and merge the header po with any comments at the start of the properties file
         appendedheader = False
@@ -53,16 +57,20 @@ class prop2po:
                     thetargetfile.addunit(targetheader)
                 appendedheader = True
             if pounit is not None:
-                pounit.addnote("".join(waitingcomments).rstrip(), "developer", position="prepend")
+                pounit.addnote("\n".join(waitingcomments).rstrip(), "developer", position="prepend")
                 waitingcomments = []
                 thetargetfile.addunit(pounit)
         thetargetfile.removeduplicates(duplicatestyle)
         return thetargetfile
 
-    def mergestore(self, origpropfile, translatedpropfile, blankmsgstr=False, duplicatestyle="msgctxt"):
+    def mergestore(self, origpropfile, translatedpropfile, personality="java", blankmsgstr=False, duplicatestyle="msgctxt"):
         """converts two .properties files to a .po file..."""
+        self.personality = personality
         thetargetfile = po.pofile()
-        targetheader = thetargetfile.makeheader(charset="UTF-8", encoding="8bit")
+        if self.personality == "mozilla" or self.personality == "skype":
+            targetheader = thetargetfile.makeheader(charset="UTF-8", encoding="8bit", x_accelerator_marker="&")
+        else:
+            targetheader = thetargetfile.makeheader(charset="UTF-8", encoding="8bit")
         targetheader.addnote("extracted from %s, %s" % (origpropfile.filename, translatedpropfile.filename), "developer")
         translatedpropfile.makeindex()
         # we try and merge the header po with any comments at the start of the properties file
@@ -96,7 +104,7 @@ class prop2po:
             if origpo is not None:
                 if translatedpo is not None and not blankmsgstr:
                     origpo.target = translatedpo.source
-                origpo.addnote("".join(waitingcomments).rstrip(), "developer", position="prepend")
+                origpo.addnote(u"".join(waitingcomments).rstrip(), "developer", position="prepend")
                 waitingcomments = []
                 thetargetfile.addunit(origpo)
             elif translatedpo is not None:
@@ -115,24 +123,28 @@ class prop2po:
             for comment in propunit.comments:
                 if "DONT_TRANSLATE" in comment:
                     return "discard"
-            pounit.addnote("".join(propunit.comments).rstrip(), commenttype)
+            pounit.addnote(u"".join(propunit.getnotes()).rstrip(), commenttype)
         # TODO: handle multiline msgid
         if propunit.isblank():
             return None
         pounit.addlocation(propunit.name)
         pounit.source = propunit.source
-        pounit.target = ""
+        pounit.target = u""
         return pounit
 
-def convertprop(inputfile, outputfile, templatefile, pot=False, duplicatestyle="msgctxt"):
+def convertmozillaprop(inputfile, outputfile, templatefile, pot=False, duplicatestyle="msgctxt"):
+    """Mozilla specific convertor function"""
+    return convertprop(inputfile, outputfile, templatefile, personality="mozilla", pot=pot, duplicatestyle=duplicatestyle)
+
+def convertprop(inputfile, outputfile, templatefile, personality="java", pot=False, duplicatestyle="msgctxt"):
     """reads in inputfile using properties, converts using prop2po, writes to outputfile"""
-    inputstore = properties.propfile(inputfile)
+    inputstore = properties.propfile(inputfile, personality)
     convertor = prop2po()
     if templatefile is None:
-        outputstore = convertor.convertstore(inputstore, duplicatestyle=duplicatestyle)
+        outputstore = convertor.convertstore(inputstore, personality, duplicatestyle=duplicatestyle)
     else:
-        templatestore = properties.propfile(templatefile)
-        outputstore = convertor.mergestore(templatestore, inputstore, blankmsgstr=pot, duplicatestyle=duplicatestyle)
+        templatestore = properties.propfile(templatefile, personality)
+        outputstore = convertor.mergestore(templatestore, inputstore, personality, blankmsgstr=pot, duplicatestyle=duplicatestyle)
     if outputstore.isempty():
         return 0
     outputfile.write(str(outputstore))
@@ -140,10 +152,17 @@ def convertprop(inputfile, outputfile, templatefile, pot=False, duplicatestyle="
 
 def main(argv=None):
     from translate.convert import convert
-    formats = {"properties": ("po", convertprop), ("properties", "properties"): ("po", convertprop)}
+    formats = {"properties": ("po", convertprop), 
+               ("properties", "properties"): ("po", convertprop),
+               "lang": ("po", convertprop),
+               ("lang", "lang"): ("po", convertprop),}
     parser = convert.ConvertOptionParser(formats, usetemplates=True, usepots=True, description=__doc__)
+    parser.add_option("", "--personality", dest="personality", default="java", type="choice",
+            choices=["java", "mozilla", "skype"],
+            help="set the input behaviour: java (default), mozilla, skype", metavar="TYPE")
     parser.add_duplicates_option()
     parser.passthrough.append("pot")
+    parser.passthrough.append("personality")
     parser.run(argv)
 
 if __name__ == '__main__':
