@@ -1,23 +1,22 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# 
-# Copyright 2007-2008 Zuza Software Foundation
-# 
-# This file is part of translate.
 #
-# translate is free software; you can redistribute it and/or modify
+# Copyright 2007-2009 Zuza Software Foundation
+#
+# This file is part of the Translate Toolkit.
+#
+# This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
-# translate is distributed in the hope that it will be useful,
+#
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 """convert .rc files to Gettext PO localization files"""
 
@@ -26,7 +25,7 @@ from translate.storage import po
 from translate.storage import rc
 
 class rc2po:
-    """convert a .rc file to a .po file for handling the translation..."""
+    """Convert a .rc file to a .po file for handling the translation."""
     def __init__(self, charset=None):
         self.charset = charset
 
@@ -81,14 +80,14 @@ class rc2po:
         output_unit.target = ""
         return output_unit
 
-def convertrc(input_file, output_file, template_file, pot=False, duplicatestyle="msgctxt", charset=None):
+def convertrc(input_file, output_file, template_file, pot=False, duplicatestyle="msgctxt", charset=None, lang=None, sublang=None):
     """reads in input_file using rc, converts using rc2po, writes to output_file"""
-    input_store = rc.rcfile(input_file)
+    input_store = rc.rcfile(input_file, lang, sublang)
     convertor = rc2po(charset=charset)
     if template_file is None:
         output_store = convertor.convert_store(input_store, duplicatestyle=duplicatestyle)
     else:
-        template_store = rc.rcfile(template_file)
+        template_store = rc.rcfile(template_file, lang, sublang)
         output_store = convertor.merge_store(template_store, input_store, blankmsgstr=pot, duplicatestyle=duplicatestyle)
     if output_store.isempty():
         return 0
@@ -97,17 +96,24 @@ def convertrc(input_file, output_file, template_file, pot=False, duplicatestyle=
 
 def main(argv=None):
     from translate.convert import convert
-    formats = {"rc": ("po", convertrc), ("rc", "rc"): ("po", convertrc), 
+    formats = {"rc": ("po", convertrc), ("rc", "rc"): ("po", convertrc),
                "nls": ("po", convertrc), ("nls", "nls"): ("po", convertrc)}
     parser = convert.ConvertOptionParser(formats, usetemplates=True, usepots=True, description=__doc__)
     DEFAULTCHARSET = "cp1252"
     parser.add_option("", "--charset", dest="charset", default=DEFAULTCHARSET,
         help="charset to use to decode the RC files (default: %s)" % DEFAULTCHARSET, metavar="CHARSET")
+    DEFAULTLANG = "LANG_ENGLISH"
+    parser.add_option("-l", "--lang", dest="lang", default=DEFAULTLANG,
+        help="LANG entry (default: %s)" % DEFAULTLANG, metavar="LANG")
+    DEFAULTSUBLANG = "SUBLANG_DEFAULT"
+    parser.add_option("", "--sublang", dest="sublang", default=DEFAULTSUBLANG,
+        help="SUBLANG entry (default: %s)" % DEFAULTSUBLANG, metavar="SUBLANG")
     parser.add_duplicates_option()
     parser.passthrough.append("pot")
     parser.passthrough.append("charset")
+    parser.passthrough.append("lang")
+    parser.passthrough.append("sublang")
     parser.run(argv)
 
 if __name__ == '__main__':
     main()
-
