@@ -137,7 +137,8 @@ class SeriousFilterFailure(FilterFailure):
 #the property/tag that is specified as None. A non-None value of "value"
 #indicates that the value of the attribute must be taken into account.
 common_ignoretags = [(None, "xml-lang", None)]
-common_canchangetags = [("img", "alt", None)]
+common_canchangetags = [("img", "alt", None), (None, "title", None)]
+# Actually the title tag is allowed on many tags in HTML (but probably not all)
 
 class CheckerConfig(object):
     """object representing the configuration of a checker"""
@@ -280,6 +281,8 @@ class UnitChecker(object):
     def setsuggestionstore(self, store):
         """Sets the filename that a checker should use for evaluating suggestions."""
         self.suggestion_store = store
+        if self.suggestion_store:
+            self.suggestion_store.require_index()
 
     def filtervariables(self, str1):
         """filter out variables from str1"""
@@ -385,8 +388,8 @@ class TranslationChecker(UnitChecker):
     def run_filters(self, unit):
         """Do some optimisation by caching some data of the unit for the benefit 
         of run_test()."""
-        self.str1 = data.normalized_unicode(unit.source)
-        self.str2 = data.normalized_unicode(unit.target)
+        self.str1 = data.normalized_unicode(unit.source) or u""
+        self.str2 = data.normalized_unicode(unit.target) or u""
         self.hasplural = unit.hasplural()
         self.locations = unit.getlocations()
         return super(TranslationChecker, self).run_filters(unit)
@@ -450,11 +453,14 @@ class StandardChecker(TranslationChecker):
 
     def unchanged(self, str1, str2):
         """checks whether a translation is basically identical to the original string"""
-        str1 = self.removevariables(self.filteraccelerators(str1)).strip()
-        str2 = self.removevariables(self.filteraccelerators(str2)).strip()
+        str1 = self.filteraccelerators(self.removevariables(str1)).strip()
+        str2 = self.filteraccelerators(self.removevariables(str2)).strip()
         if len(str1) < 2:
             return True
-        if str1.isupper() and str1 == str2:
+        # If the whole string is upperase, or nothing in the string can go
+        # towards uppercase, let's assume there is nothing translatable
+        # TODO: reconsider
+        if (str1.isupper() or str1.upper() == str1) and str1 == str2:
             return True
         if self.config.notranslatewords:
             words1 = str1.split()
@@ -463,7 +469,9 @@ class StandardChecker(TranslationChecker):
             #   if len(words1) == 1 and words1[0] in self.config.notranslatewords:
             #why do we only test for one notranslate word?
                 return True
-        if str1.isalpha() and str1.lower() == str2.lower():
+        # we could also check for things like str1.isnumeric(), but the test
+        # above (str1.upper() == str1) makes this unnecessary
+        if str1.lower() == str2.lower():
             raise FilterFailure(u"please translate")
         return True
 
@@ -508,7 +516,6 @@ class StandardChecker(TranslationChecker):
         else:
             return True
 
-
     def singlequoting(self, str1, str2):
         """checks whether singlequoting is consistent between the two strings"""
         str1 = self.filterwordswithpunctuation(self.filteraccelerators(self.filtervariables(str1)))
@@ -522,7 +529,7 @@ class StandardChecker(TranslationChecker):
         str1 = self.config.lang.punctranslate(str1)
         str2 = self.filteraccelerators(self.filtervariables(str2))
         str2 = self.filterxml(str2)
-        return helpers.countsmatch(str1, str2, (u'"', u'""', u'\\"', u"«", u"»"))
+        return helpers.countsmatch(str1, str2, (u'"', u'""', u'\\"', u"«", u"»", u"“", u"”"))
 
     def doublespacing(self, str1, str2):
         """checks for bad double-spaces by comparing to original"""
@@ -699,14 +706,11 @@ class StandardChecker(TranslationChecker):
 
     def startwhitespace(self, str1, str2):
         """checks whether whitespace at the beginning of the strings matches"""
-        str1 = self.filteraccelerators(self.filtervariables(str1))
-        str2 = self.filteraccelerators(self.filtervariables(str2))
         return helpers.funcmatch(str1, str2, decoration.spacestart)
 
     def endwhitespace(self, str1, str2):
         """checks whether whitespace at the end of the strings matches"""
-        str1 = self.filteraccelerators(self.filtervariables(str1))
-        str2 = self.filteraccelerators(self.filtervariables(str2))
+        str1 = self.config.lang.punctranslate(str1)
         return helpers.funcmatch(str1, str2, decoration.spaceend)
 
     def startpunc(self, str1, str2):
@@ -757,6 +761,8 @@ class StandardChecker(TranslationChecker):
 
     def sentencecount(self, str1, str2):
         """checks that the number of sentences in both strings match"""
+        str1 = self.filteraccelerators(str1)
+        str2 = self.filteraccelerators(str2)
         sentences1 = len(self.config.sourcelang.sentences(str1))
         sentences2 = len(self.config.lang.sentences(str2))
         if not sentences1 == sentences2:
@@ -841,7 +847,7 @@ class StandardChecker(TranslationChecker):
         without_newlines = "\n".join(str2.split("\n"))
         words = self.filteraccelerators(self.removevariables(without_newlines)).replace(u".", u"").lower().split()
         for word in words:
-            if word == lastword:
+            if word == lastword and word not in self.config.lang.validdoublewords:
                 raise FilterFailure(u"The word '%s' is repeated" % word)
             lastword = word
         return True
@@ -974,7 +980,7 @@ class StandardChecker(TranslationChecker):
             # hack to ignore hyphenisation rules
             if word in suggestions:
                 continue
-            messages.append(u"check spelling of %s (could be %s)" % (word, u" / ".join(suggestions)))
+            messages.append(u"check spelling of %s (could be %s)" % (word, u" / ".join(suggestions[:5])))
         if messages:
             raise FilterFailure(messages)
         return True
@@ -1015,7 +1021,14 @@ class StandardChecker(TranslationChecker):
                                     "emails", "simpleplurals", "urls", "printf",
                                     "tabs", "newlines", "functions", "options"),
                    "purepunc":      ("startcaps", "options"),
-                   "startcaps":     ("simplecaps",),
+                   # This is causing some problems since Python 2.6, as
+                   # startcaps is now seen as an important one to always execute
+                   # and could now be done before it is blocked by a failing
+                   # "untranslated" or "blank" test. This is probably happening
+                   # due to slightly different implementation of the internal
+                   # dict handling since Python 2.6. We should never have relied
+                   # on this ordering anyway.
+                   #"startcaps":     ("simplecaps",),
                    "endwhitespace": ("endpunc",),
                    "startwhitespace":("startpunc",),
                    "unchanged":     ("doublewords",), 
@@ -1060,8 +1073,15 @@ class MozillaChecker(StandardChecker):
         checkerconfig.update(mozillaconfig)
         StandardChecker.__init__(self, **kwargs)
 
+    def credits(self, str1, str2):
+        """checks for messages containing translation credits instead of normal translations."""
+        for location in self.locations:
+            if location in ['MOZ_LANGPACK_CONTRIBUTORS', 'credit.translation']:
+                return False
+        return True
+
 drupalconfig = CheckerConfig(
-    varmatches = [("%", None), ("@", None)],
+    varmatches = [("%", None), ("@", None), ("!", None)],
     )
 
 class DrupalChecker(StandardChecker):
@@ -1161,8 +1181,7 @@ class StandardUnitChecker(UnitChecker):
         self.suggestion_store = getattr(self, 'suggestion_store', None)
         suggestions = []
         if self.suggestion_store:
-            source = unit.source
-            suggestions = [unit for unit in self.suggestion_store.units if unit.source == source]
+            suggestions = self.suggestion_store.findunits(unit.source)
         elif xliff and isinstance(unit, xliff.xliffunit):
             # TODO: we probably want to filter them somehow
             suggestions = unit.getalttrans()
