@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 from translate.filters import checks
 from translate.lang import data
-from translate.storage import po 
+from translate.storage import po
 
 def strprep(str1, str2, message=None):
     return data.normalized_unicode(str1), data.normalized_unicode(str2), data.normalized_unicode(message)
@@ -81,37 +81,38 @@ def test_messages():
     assert fails(stdchecker.validchars, "Some unexpected characters", "©", "invalid chars: '©' (\\u00a9)")
     stdchecker = checks.StandardChecker()
     assert fails_serious(stdchecker.escapes, r"A tab", r"'n Ṱab\t", r"""escapes in original () don't match escapes in translation ('Ṱab\t')""")
-    
+
 def test_accelerators():
     """tests accelerators"""
     stdchecker = checks.StandardChecker(checks.CheckerConfig(accelmarkers="&"))
-    assert passes(stdchecker.accelerators, "&File", "&Fayile") 
-    assert fails(stdchecker.accelerators, "&File", "Fayile") 
-    assert fails(stdchecker.accelerators, "File", "&Fayile") 
-    assert passes(stdchecker.accelerators, "Mail && News", "Pos en Nuus") 
-    assert fails(stdchecker.accelerators, "Mail &amp; News", "Pos en Nuus") 
+    assert passes(stdchecker.accelerators, "&File", "&Fayile")
+    assert fails(stdchecker.accelerators, "&File", "Fayile")
+    assert fails(stdchecker.accelerators, "File", "&Fayile")
+    assert passes(stdchecker.accelerators, "Mail && News", "Pos en Nuus")
+    assert fails(stdchecker.accelerators, "Mail &amp; News", "Pos en Nuus")
     assert passes(stdchecker.accelerators, "&Allow", u'&\ufeb2\ufee3\ufe8e\ufea3')
-    assert fails(stdchecker.accelerators, "Open &File", "Vula& Ifayile") 
+    assert fails(stdchecker.accelerators, "Open &File", "Vula& Ifayile")
     kdechecker = checks.KdeChecker()
-    assert passes(kdechecker.accelerators, "&File", "&Fayile") 
-    assert fails(kdechecker.accelerators, "&File", "Fayile") 
-    assert fails(kdechecker.accelerators, "File", "&Fayile") 
+    assert passes(kdechecker.accelerators, "&File", "&Fayile")
+    assert fails(kdechecker.accelerators, "&File", "Fayile")
+    assert fails(kdechecker.accelerators, "File", "&Fayile")
     gnomechecker = checks.GnomeChecker()
-    assert passes(gnomechecker.accelerators, "_File", "_Fayile") 
-    assert fails(gnomechecker.accelerators, "_File", "Fayile") 
-    assert fails(gnomechecker.accelerators, "File", "_Fayile") 
-    assert fails(gnomechecker.accelerators, "_File", "_Fayil_e") 
+    assert passes(gnomechecker.accelerators, "_File", "_Fayile")
+    assert fails(gnomechecker.accelerators, "_File", "Fayile")
+    assert fails(gnomechecker.accelerators, "File", "_Fayile")
+    assert fails(gnomechecker.accelerators, "_File", "_Fayil_e")
     mozillachecker = checks.MozillaChecker()
-    assert passes(mozillachecker.accelerators, "&File", "&Fayile") 
-    assert fails_serious(mozillachecker.accelerators, "&File", "Fayile") 
-    assert fails_serious(mozillachecker.accelerators, "File", "&Fayile") 
-    assert passes(mozillachecker.accelerators, "Mail &amp; News", "Pos en Nuus") 
-    assert fails_serious(mozillachecker.accelerators, "Mail &amp; News", "Pos en &Nuus") 
-    assert fails_serious(mozillachecker.accelerators, "&File", "Fayile") 
+    assert passes(mozillachecker.accelerators, "&File", "&Fayile")
+    assert passes(mozillachecker.accelerators, "Warn me if this will disable any of my add&-ons", "&Waarsku my as dit enige van my byvoegings sal deaktiveer")
+    assert fails_serious(mozillachecker.accelerators, "&File", "Fayile")
+    assert fails_serious(mozillachecker.accelerators, "File", "&Fayile")
+    assert passes(mozillachecker.accelerators, "Mail &amp; News", "Pos en Nuus")
+    assert fails_serious(mozillachecker.accelerators, "Mail &amp; News", "Pos en &Nuus")
+    assert fails_serious(mozillachecker.accelerators, "&File", "Fayile")
     ooochecker = checks.OpenOfficeChecker()
-    assert passes(ooochecker.accelerators, "~File", "~Fayile") 
-    assert fails(ooochecker.accelerators, "~File", "Fayile") 
-    assert fails(ooochecker.accelerators, "File", "~Fayile") 
+    assert passes(ooochecker.accelerators, "~File", "~Fayile")
+    assert fails(ooochecker.accelerators, "~File", "Fayile")
+    assert fails(ooochecker.accelerators, "File", "~Fayile")
 
     # We don't want an accelerator for letters with a diacritic
     assert fails(ooochecker.accelerators, "F~ile", "L~êer")
@@ -130,7 +131,6 @@ def xtest_acceleratedvariables():
     ooochecker = checks.OpenOfficeChecker()
     assert fails(ooochecker.acceleratedvariables, "%PRODUCTNAME% ~Options", "~%PRODUCTNAME% Ikhetho")
     assert passes(ooochecker.acceleratedvariables, "%PRODUCTNAME% ~Options", "%PRODUCTNAME% ~Ikhetho")
-    
 
 def test_acronyms():
     """tests acronyms"""
@@ -186,10 +186,10 @@ def test_doublequoting():
     assert fails(frchecker.doublequoting, "Do \"this\"", "Do \"this\"")
     assert passes(frchecker.doublequoting, "Do \"this\"", "Do « this »")
     assert fails(frchecker.doublequoting, "Do \"this\"", "Do « this » « this »")
-    
+
     vichecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage="vi"))
     assert passes(vichecker.doublequoting, 'Save "File"', u"Lưu « Tập tin »")
-    
+
 def test_doublespacing():
     """tests double spacing"""
     stdchecker = checks.StandardChecker()
@@ -209,6 +209,11 @@ def test_doublewords():
     # Double variables are not an error
     stdchecker = checks.StandardChecker(checks.CheckerConfig(varmatches=[("%", 1)]))
     assert passes(stdchecker.doublewords, "%s %s installation", "tsenyo ya %s %s")
+    # In some language certain double words are not errors
+    st_checker = checks.StandardChecker(checks.CheckerConfig(targetlanguage="st"))
+    assert passes(st_checker.doublewords, "Color to draw the name of a message you sent.", "Mmala wa ho taka bitso la molaetsa oo o o rometseng.")
+    assert passes(st_checker.doublewords, "Ten men", "Banna ba ba leshome")
+    assert passes(st_checker.doublewords, "Give SARS the tax", "Lekgetho le le fe SARS")
 
 def test_endpunc():
     """tests endpunc"""
@@ -234,12 +239,19 @@ def test_endpunc():
     stdchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage='vi'))
     assert passes(stdchecker.endpunc, "Do you want to delete the XX dialog?", u"Bạn có muốn xoá hộp thoại XX không?")
 
+    stdchecker = checks.StandardChecker(checks.CheckerConfig(targetlanguage='fr'))
+    assert passes(stdchecker.endpunc, "Header:", u"En-tête :")
+
 def test_endwhitespace():
     """tests endwhitespace"""
     stdchecker = checks.StandardChecker()
     assert passes(stdchecker.endwhitespace, "A setence. ", "I'm correct. ")
     assert fails(stdchecker.endwhitespace, "A setence. ", "'I'm incorrect.")
 
+    zh_checker = checks.StandardChecker(checks.CheckerConfig(targetlanguage='zh'))
+    # This should pass since the space is not needed in Chinese
+    assert passes(zh_checker.endwhitespace, "Init. Limit: ", "起始时间限制：")
+
 def test_escapes():
     """tests escapes"""
     stdchecker = checks.StandardChecker()
@@ -293,7 +305,7 @@ def test_filepaths():
     stdchecker = checks.StandardChecker()
     assert passes(stdchecker.filepaths, "%s to the file /etc/hosts on your system.", "%s na die leer /etc/hosts op jou systeem.")
     assert fails(stdchecker.filepaths, "%s to the file /etc/hosts on your system.", "%s na die leer /etc/gasheer op jou systeem.")
-    
+
 def test_kdecomments():
     """tests kdecomments"""
     stdchecker = checks.StandardChecker()
@@ -610,21 +622,27 @@ def test_startwhitespace():
 def test_unchanged():
     """tests unchanged entries"""
     stdchecker = checks.StandardChecker(checks.CheckerConfig(accelmarkers="&"))
-    assert fails(stdchecker.unchanged, "Unchanged", "Unchanged") 
-    assert fails(stdchecker.unchanged, "&Unchanged", "Un&changed") 
-    assert passes(stdchecker.unchanged, "Unchanged", "Changed") 
-    assert passes(stdchecker.unchanged, "1234", "1234") 
+    assert fails(stdchecker.unchanged, "Unchanged", "Unchanged")
+    assert fails(stdchecker.unchanged, "&Unchanged", "Un&changed")
+    assert passes(stdchecker.unchanged, "Unchanged", "Changed")
+    assert passes(stdchecker.unchanged, "1234", "1234")
     assert passes(stdchecker.unchanged, "2×2", "2×2") # bug 178, description item 14
-    assert passes(stdchecker.unchanged, "I", "I") 
+    assert passes(stdchecker.unchanged, "I", "I")
     assert passes(stdchecker.unchanged, "   ", "   ")  # bug 178, description item 5
     assert passes(stdchecker.unchanged, "???", "???")  # bug 178, description item 15
     assert passes(stdchecker.unchanged, "&ACRONYM", "&ACRONYM") # bug 178, description item 7
     assert passes(stdchecker.unchanged, "F1", "F1") # bug 178, description item 20
+    assert fails(stdchecker.unchanged, "Two words", "Two words")
+    #TODO: this still fails
+#    assert passes(stdchecker.unchanged, "NOMINAL", "NOMİNAL")
+    gnomechecker = checks.GnomeChecker()
+    assert fails(gnomechecker.unchanged, "Entity references, such as &amp; and &#169;", "Entity references, such as &amp; and &#169;")
     # Variable only and variable plus punctuation messages should be ignored
     mozillachecker = checks.MozillaChecker()
-    assert passes(mozillachecker.unchanged, "$ProgramName$", "$ProgramName$") 
+    assert passes(mozillachecker.unchanged, "$ProgramName$", "$ProgramName$")
     assert passes(mozillachecker.unchanged, "$file$ : $dir$", "$file$ : $dir$") # bug 178, description item 13
-    assert fails(mozillachecker.unchanged, "$file$ in $dir$", "$file$ in $dir$") 
+    assert fails(mozillachecker.unchanged, "$file$ in $dir$", "$file$ in $dir$")
+    assert passes(mozillachecker.unchanged, "&brandShortName;", "&brandShortName;")
     # Don't translate words should be ignored
     stdchecker = checks.StandardChecker(checks.CheckerConfig(notranslatewords=["Mozilla"]))
     assert passes(stdchecker.unchanged, "Mozilla", "Mozilla") # bug 178, description item 10
@@ -763,6 +781,8 @@ def test_xmltags():
     assert passes(stdchecker.xmltags, "Click <img src=\"img.jpg\">here</img>", "Klik <img src=\"img.jpg\">hier</img>")
     assert fails(stdchecker.xmltags, "Click <img src=\"image.jpg\">here</img>", "Klik <img src=\"prent.jpg\">hier</img>")
     assert passes(stdchecker.xmltags, "Click <img src=\"img.jpg\" alt=\"picture\">here</img>", "Klik <img src=\"img.jpg\" alt=\"prentjie\">hier</img>")
+    assert passes(stdchecker.xmltags, "Click <a title=\"tip\">here</a>", "Klik <a title=\"wenk\">hier</a>")
+    assert passes(stdchecker.xmltags, "Click <div title=\"tip\">here</div>", "Klik <div title=\"wenk\">hier</div>")
     assert passes(stdchecker.xmltags, "Start with the &lt;start&gt; tag", "Begin met die &lt;begin&gt;")
 
     assert fails(stdchecker.xmltags, "Click <a href=\"page.html\">", "Klik <a hverw=\"page.html\">")
@@ -871,20 +891,27 @@ def test_nplurals():
 def test_credits():
     """tests credits"""
     stdchecker = checks.StandardChecker()
-    assert passes(stdchecker.credits, "File", "iFayile") 
-    assert passes(stdchecker.credits, "&File", "&Fayile") 
-    assert passes(stdchecker.credits, "translator-credits", "Ekke, ekke!") 
-    assert passes(stdchecker.credits, "Your names", "Ekke, ekke!") 
-    assert passes(stdchecker.credits, "ROLES_OF_TRANSLATORS", "Ekke, ekke!") 
+    assert passes(stdchecker.credits, "File", "iFayile")
+    assert passes(stdchecker.credits, "&File", "&Fayile")
+    assert passes(stdchecker.credits, "translator-credits", "Ekke, ekke!")
+    assert passes(stdchecker.credits, "Your names", "Ekke, ekke!")
+    assert passes(stdchecker.credits, "ROLES_OF_TRANSLATORS", "Ekke, ekke!")
     kdechecker = checks.KdeChecker()
-    assert passes(kdechecker.credits, "File", "iFayile") 
-    assert passes(kdechecker.credits, "&File", "&Fayile") 
-    assert passes(kdechecker.credits, "translator-credits", "Ekke, ekke!") 
-    assert fails(kdechecker.credits, "Your names", "Ekke, ekke!") 
-    assert fails(kdechecker.credits, "ROLES_OF_TRANSLATORS", "Ekke, ekke!") 
+    assert passes(kdechecker.credits, "File", "iFayile")
+    assert passes(kdechecker.credits, "&File", "&Fayile")
+    assert passes(kdechecker.credits, "translator-credits", "Ekke, ekke!")
+    assert fails(kdechecker.credits, "Your names", "Ekke, ekke!")
+    assert fails(kdechecker.credits, "ROLES_OF_TRANSLATORS", "Ekke, ekke!")
     gnomechecker = checks.GnomeChecker()
-    assert passes(gnomechecker.credits, "File", "iFayile") 
-    assert passes(gnomechecker.credits, "&File", "&Fayile") 
-    assert fails(gnomechecker.credits, "translator-credits", "Ekke, ekke!") 
-    assert passes(gnomechecker.credits, "Your names", "Ekke, ekke!") 
-    assert passes(gnomechecker.credits, "ROLES_OF_TRANSLATORS", "Ekke, ekke!") 
+    assert passes(gnomechecker.credits, "File", "iFayile")
+    assert passes(gnomechecker.credits, "&File", "&Fayile")
+    assert fails(gnomechecker.credits, "translator-credits", "Ekke, ekke!")
+    assert passes(gnomechecker.credits, "Your names", "Ekke, ekke!")
+    assert passes(gnomechecker.credits, "ROLES_OF_TRANSLATORS", "Ekke, ekke!")
+
+def test_gconf():
+    """test GNOME gconf errors"""
+    gnomechecker = checks.GnomeChecker()
+    assert passes(gnomechecker.gconf, 'Blah "gconf_setting"', 'Bleh "gconf_setting"')
+    assert fails(gnomechecker.gconf, 'Blah "gconf_setting"', 'Bleh "gconf_steling"')
+
