@@ -101,7 +101,7 @@ class Common(object):
     # some code probably depends on these for unrecognised languages
     
     listseperator = u", "
-    """This string is used to seperate lists of textual elements. Most 
+    """This string is used to separate lists of textual elements. Most 
     languages probably can stick with the default comma, but Arabic and some
     Asian languages might want to override this."""
     
@@ -175,6 +175,10 @@ class Common(object):
     but normally they would be at least precomposed characters. All characters,
     lower and upper, are included in the list."""
 
+    validdoublewords = []
+    """Some languages allow double words in certain cases.  This is a dictionary
+    of such words."""
+
     def __new__(cls, code):
         """This returns the language class for the given code, following a 
         singleton like approach (only one object per language)."""
@@ -197,6 +201,10 @@ class Common(object):
             pass
         return language
 
+    def __deepcopy__(self, memo={}):
+        memo[id(self)] = self
+        return self
+
     def __repr__(self):
         """Give a simple string representation without address information to 
         be able to store it in text for comparison later."""
@@ -211,13 +219,22 @@ class Common(object):
 #        TODO: look at po::escapeforpo() for performance idea
         if not text:
             return text
+        ellipses_end = text.endswith(u"...")
+        if ellipses_end:
+            text = text[:-3]
         for source, target in cls.puncdict.iteritems():
             text = text.replace(source, target)
+        if ellipses_end:
+            if u"..." in cls.puncdict:
+                text += cls.puncdict[u"..."]
+            else:
+                text += u"..."
         # Let's account for cases where a punctuation symbol plus a space is 
         # replaced, but the space won't exist at the end of a message.
         # As a simple improvement for messages ending in ellipses (...), we
         # test that the last character is different from the second last
-        if (text[-1] + u" " in cls.puncdict) and (text[-2] != text[-1]):
+        # This is only relevant if the string has two characters or more
+        if (text[-1] + u" " in cls.puncdict) and (len(text) < 2 or text[-2] != text[-1]):
             text = text[:-1] + cls.puncdict[text[-1] + u" "].rstrip()
         return text
     punctranslate = classmethod(punctranslate)
@@ -254,9 +271,9 @@ class Common(object):
                     text = text[-extra:]
             return text
         expanded = []
-        for subtext in text.split("\n\n"):
+        for subtext in text.split(u"\n\n"):
             expanded.append(alter_it(subtext))
-        text = "\n\n".join(expanded)
+        text = u"\n\n".join(expanded)
         return text
 
     def character_iter(cls, text):
@@ -304,7 +321,7 @@ class Common(object):
         if strip: remainder = remainder.strip()
         if remainder: yield remainder
     sentence_iter = classmethod(sentence_iter)
-            
+
     def sentences(cls, text, strip=True):
         """Returns a list of senteces in text."""
         return [s for s in cls.sentence_iter(text, strip=strip)]
