@@ -35,7 +35,8 @@ class fa(common.Common):
         u",": u"،",
         u";": u"؛",
         u"?": u"؟",
-        u"%": u"٪",
+        #This causes problems with variables, so commented out for now:
+        #u"%": u"٪",
     }
 
     ignoretests = ["startcaps", "simplecaps"]
