@@ -25,13 +25,15 @@ __all__ = ['get_abs_data_filename']
 import sys
 import os
 
-def get_abs_data_filename(path_parts, basedirs=[]):
+def get_abs_data_filename(path_parts, basedirs=None):
     """Get the absolute path to the given file- or directory name in the current
         running application's data directory.
 
         @type  path_parts: list
         @param path_parts: The path parts that can be joined by os.path.join().
         """
+    if basedirs is None:
+        basedirs = []
 
     if isinstance(path_parts, str):
         path_parts = [path_parts]
