@@ -79,7 +79,7 @@ class RecursiveOptionParser(optparse.OptionParser, object):
 
         """
 
-        optparse.OptionParser.__init__(self, version="%prog "+__version__.ver, description=description)
+        optparse.OptionParser.__init__(self, version="%prog "+__version__.sver, description=description)
         self.setmanpageoption()
         self.setprogressoptions()
         self.seterrorleveloptions()
@@ -135,6 +135,10 @@ class RecursiveOptionParser(optparse.OptionParser, object):
         file.write(self.format_manpage())
 
     def setpsycooption(self):
+        try:
+            import psyco
+        except Exception:
+            return
         psycomodes = ["none", "full", "profile"]
         psycooption = optparse.Option(None, "--psyco", dest="psyco", default=None,
             choices=psycomodes, metavar="MODE",
@@ -144,7 +148,7 @@ class RecursiveOptionParser(optparse.OptionParser, object):
     def usepsyco(self, options):
         # options.psyco == None means the default, which is "full", but don't give a warning...
         # options.psyco == "none" means don't use psyco at all...
-        if options.psyco == "none":
+        if getattr(options, "psyco", "none") == "none":
             return
         try:
             import psyco
@@ -248,9 +252,12 @@ class RecursiveOptionParser(optparse.OptionParser, object):
             if not isinstance(outputoptions, tuple) or len(outputoptions) != 2:
                 raise ValueError("output options must be tuples of length 2")
             outputformat, processor = outputoptions
-            if not inputformat in inputformats: inputformats.append(inputformat)
-            if not outputformat in outputformats: outputformats.append(outputformat)
-            if not templateformat in templateformats: templateformats.append(templateformat)
+            if not inputformat in inputformats:
+                inputformats.append(inputformat)
+            if not outputformat in outputformats:
+                outputformats.append(outputformat)
+            if not templateformat in templateformats:
+                templateformats.append(templateformat)
             self.outputoptions[(inputformat, templateformat)] = (outputformat, processor)
         self.inputformats = inputformats
         inputformathelp = self.getformathelp(inputformats)
@@ -624,7 +631,13 @@ class RecursiveOptionParser(optparse.OptionParser, object):
         return inputfiles
 
     def splitext(self, pathname):
-        """splits into name and ext, and removes the extsep"""
+        """Splits L{pathname} into name and ext, and removes the extsep
+    
+        @param pathname: A file path
+        @type pathname: string
+        @return: root, ext
+        @rtype: tuple
+        """
         root, ext = os.path.splitext(pathname)
         ext = ext.replace(os.extsep, "", 1)
         return (root, ext)
@@ -644,8 +657,10 @@ class RecursiveOptionParser(optparse.OptionParser, object):
 
     def gettemplatename(self, options, inputname):
         """gets an output filename based on the input filename"""
-        if not self.usetemplates: return None
-        if not inputname or not options.recursivetemplate: return options.template
+        if not self.usetemplates:
+            return None
+        if not inputname or not options.recursivetemplate:
+            return options.template
         inputbase, inputext = self.splitinputext(inputname)
         if options.template:
             for inputext1, templateext1 in options.outputoptions:
@@ -669,7 +684,8 @@ class RecursiveOptionParser(optparse.OptionParser, object):
 
     def getoutputname(self, options, inputname, outputformat):
         """gets an output filename based on the input filename"""
-        if not inputname or not options.recursiveoutput: return options.output
+        if not inputname or not options.recursiveoutput:
+            return options.output
         inputbase, inputext = self.splitinputext(inputname)
         outputname = inputbase
         if outputformat:
