@@ -119,6 +119,7 @@ def extractwithoutquotes(source, startdelim, enddelim, escape=None, startinstrin
         enddelim_places = startdelim_places[:]
     else:
         enddelim_places = find_all(source, enddelim)
+    #hell slow because it is called far too often
     if escape is not None:
         lenescape = len(escape)
         escape_places = find_all(source, escape)
@@ -241,7 +242,7 @@ def htmlentitydecode(source):
 
 def javapropertiesencode(source):
     """encodes source in the escaped-unicode encoding used by Java .properties files"""
-    output = ""
+    output = u""
     for char in source:
         charnum = ord(char)
         if char in controlchars:
@@ -249,12 +250,12 @@ def javapropertiesencode(source):
         elif 0 <= charnum < 128:
             output += str(char)
         else:
-            output += "\\u%04X" % charnum
+            output += u"\\u%04X" % charnum
     return output
 
 def mozillapropertiesencode(source):
     """encodes source in the escaped-unicode encoding used by Mozilla .properties files"""
-    output = ""
+    output = u""
     for char in source:
         charnum = ord(char)
         if char in controlchars:
@@ -267,12 +268,13 @@ propertyescapes = {
     # escapes that are self-escaping
     "\\": "\\", "'": "'", '"': '"',
     # control characters that we keep
-    "b": "\b", "f": "\f", "t": "\t", "n": "\n", "v": "\v", "a": "\a"
+    "f": "\f", "n": "\n", "r": "\r", "t": "\t",
     }
 
 controlchars = {
     # the reverse of the above...
-    "\b": "\\b", "\f": "\\f", "\t": "\\t", "\n": "\\n", "\v": "\\v"
+    "\\": "\\\\",
+    "\f": "\\f", "\n": "\\n", "\r": "\\r", "\t": "\\t"
     }
 
 def escapecontrols(source):
@@ -281,15 +283,17 @@ def escapecontrols(source):
         source = source.replace(key, value)
     return source
 
-def mozillapropertiesdecode(source):
-    """decodes source from the escaped-unicode encoding used by mozilla .properties files"""
+def propertiesdecode(source):
+    """Decodes source from the escaped-unicode encoding used by .properties files.
+
+    Java uses Latin1 by default, and Mozilla uses UTF-8 by default."""
     # since the .decode("unicode-escape") routine decodes everything, and we don't want to
     # we reimplemented the algorithm from Python Objects/unicode.c in Python here
     # and modified it to retain escaped control characters
     output = u""
     s = 0
     if isinstance(source, str):
-        source = source.decode("utf-8")
+        source = source.decode(encoding)
     def unichr2(i):
         """Returns a Unicode string of one character with ordinal 32 <= i, otherwise an escaped control character"""
         if 32 <= i:
@@ -352,7 +356,7 @@ def mozillapropertiesdecode(source):
             output += unicodedata.lookup(name)
             s = e + 1
         else:
-            output += "\\" + c
+            output += c # Drop any \ that we don't specifically handle
     return output
 
 def quotestr(source, escapeescapes=0):
