@@ -26,11 +26,9 @@ interface for the PyLucene (v2.x) indexing engine
 take a look at PyLuceneIndexer1.py for the PyLucene v1.x interface
 """
 
-__revision__ = "$Id: PyLuceneIndexer.py 8506 2008-09-27 09:11:32Z dwaynebailey $"
+__revision__ = "$Id: PyLuceneIndexer.py 13070 2009-11-13 16:47:01Z alaaosh $"
 
 import CommonIndexer
-# TODO: replace this dependency on the jToolkit
-import jToolkit.glock
 import tempfile
 import re
 import os
@@ -40,14 +38,14 @@ import time
 # remember the type of the detected package (compiled with jcc (>=v2.3) or
 # with gcj (<=v2.2)
 try:
-	import PyLucene
-        _COMPILER = 'gcj'
+    import PyLucene
+    _COMPILER = 'gcj'
 except ImportError:
-	# if this fails, then there is no pylucene installed
-	import lucene
-	PyLucene = lucene
-	PyLucene.initVM(PyLucene.CLASSPATH)
-        _COMPILER = 'jcc'
+    # if this fails, then there is no pylucene installed
+    import lucene
+    PyLucene = lucene
+    PyLucene.initVM(PyLucene.CLASSPATH)
+    _COMPILER = 'jcc'
 
 
 UNNAMED_FIELD_NAME = "FieldWithoutAName"
@@ -119,33 +117,30 @@ class PyLuceneDatabase(CommonIndexer.CommonDatabase):
                 raise OSError("Indexer: failed to open or create a Lucene" \
                         + " database (%s): %s" % (self.location, err_msg))
         # the indexer is initialized - now we prepare the searcher
-        # create a lock for the database directory - to be used later
-        lockname = os.path.join(tempfile.gettempdir(),
-                re.sub("\W", "_", self.location))
-        self.dir_lock = jToolkit.glock.GlobalLock(lockname)
         # windows file locking seems inconsistent, so we try 10 times
         numtries = 0
-        self.dir_lock.acquire(blocking=True)
+        #self.dir_lock.acquire(blocking=True)
         # read "self.reader", "self.indexVersion" and "self.searcher"
         try:
             while numtries < 10:
                 try:
                     self.reader = PyLucene.IndexReader.open(self.location)
                     self.indexVersion = self.reader.getCurrentVersion(
-                            self.location)
+                        self.location)
                     self.searcher = PyLucene.IndexSearcher(self.reader)
                     break
                 except PyLucene.JavaError, e:
                     # store error message for possible later re-raise (below)
                     lock_error_msg = e
                     time.sleep(0.01)
-                numtries += 1
+                    numtries += 1
             else:
                 # locking failed for 10 times
                 raise OSError("Indexer: failed to lock index database" \
-                        + " (%s)" % lock_error_msg)
+                              + " (%s)" % lock_error_msg)
         finally:
-            self.dir_lock.release()
+            pass
+        #    self.dir_lock.release()
         # initialize the searcher and the reader
         self._index_refresh()
 
@@ -378,7 +373,9 @@ class PyLuceneDatabase(CommonIndexer.CommonDatabase):
         @param docid: the document ID to be deleted
         @type docid: int
         """
+        self._delete_stale_lock()
         self.reader.deleteDocument(docid)
+        self.reader.flush()
         # TODO: check the performance impact of calling "refresh" for each id
         self._index_refresh()
 
@@ -412,12 +409,26 @@ class PyLuceneDatabase(CommonIndexer.CommonDatabase):
             result.append(fields)
         return result
 
+    def _delete_stale_lock(self):
+        if self.reader.isLocked(self.location):
+            #HACKISH: there is a lock but Lucene api can't tell us how old it
+            # is, will have to check the filesystem
+            try:
+                # in try block just in case lock disappears on us while testing it
+                stat = os.stat(os.path.join(self.location, 'write.lock'))
+                age = (time.time() - stat.st_mtime) / 60
+                if age > 15:
+                    logging.warning("stale lock found in %s, removing.", self.location)
+                    self.reader.unlock(self.reader.directory())
+            except:
+                pass
+    
     def _writer_open(self):
         """open write access for the indexing database and acquire an
         exclusive lock
         """
         if not self._writer_is_open():
-            self.dir_lock.acquire()
+            self._delete_stale_lock()
             self.writer = PyLucene.IndexWriter(self.location, self.pyl_analyzer,
                     False)
             # "setMaxFieldLength" is available since PyLucene v2
@@ -430,10 +441,9 @@ class PyLuceneDatabase(CommonIndexer.CommonDatabase):
     def _writer_close(self):
         """close indexing write access and remove the database lock"""
         if self._writer_is_open():
+            self.writer.commit()
             self.writer.close()
             self.writer = None
-        # make sure that the lock is removed
-        self.dir_lock.forcerelease()
 
     def _writer_is_open(self):
         """check if the indexing write access is currently open"""
@@ -441,12 +451,6 @@ class PyLuceneDatabase(CommonIndexer.CommonDatabase):
 
     def _index_refresh(self):
         """re-read the indexer database"""
-        try:
-            self.dir_lock.acquire(blocking=False)
-        except jToolkit.glock.GlobalLockError, e:
-            # if this fails the index is being rewritten, so we continue with
-            # our old version
-            return
         try:
             if self.reader is None or self.searcher is None:
                 self.reader = PyLucene.IndexReader.open(self.location)
@@ -462,8 +466,6 @@ class PyLuceneDatabase(CommonIndexer.CommonDatabase):
             # TODO: add some debugging output?
             #self.errorhandler.logerror("Error attempting to read index - try reindexing: "+str(e))
             pass
-        self.dir_lock.release()
-
 
 
 class PyLuceneHits(CommonIndexer.CommonEnquire):
