@@ -129,6 +129,8 @@ gpo.po_message_prev_msgctxt.restype = STRING
 gpo.po_message_prev_msgid.restype = STRING
 gpo.po_message_prev_msgid_plural.restype = STRING
 gpo.po_message_is_format.restype = c_int
+gpo.po_message_is_format.argtypes = [c_int, STRING]
+gpo.po_message_set_format.argtypes = [c_int, STRING, c_int]
 gpo.po_message_msgctxt.restype = STRING
 gpo.po_message_msgid.restype = STRING
 gpo.po_message_msgid_plural.restype = STRING
@@ -174,6 +176,8 @@ def get_libgettextpo_version():
 
 class pounit(pocommon.pounit):
     def __init__(self, source=None, encoding='utf-8', gpo_message=None):
+        self._rich_source = None
+        self._rich_target = None
         self._encoding = encoding
         if not gpo_message:
             self._gpo_message = gpo.po_message_create()
@@ -183,12 +187,6 @@ class pounit(pocommon.pounit):
         elif gpo_message:
             self._gpo_message = gpo_message
 
-    def setmsgidcomment(self, msgidcomment):
-        if msgidcomment:
-            newsource = "_: " + msgidcomment + "\n" + self.source
-            self.source = newsource
-    msgidcomment = property(None, setmsgidcomment)
-
     def setmsgid_plural(self, msgid_plural): 
         if isinstance(msgid_plural, list):
             msgid_plural = "".join(msgid_plural)
@@ -207,15 +205,15 @@ class pounit(pocommon.pounit):
                     return u""
             else:
                 return text
-        singular = remove_msgid_comments(gpo.po_message_msgid(self._gpo_message))
+        singular = remove_msgid_comments(gpo.po_message_msgid(self._gpo_message).decode(self._encoding))
         if singular:
-            multi = multistring(singular, self._encoding)
             if self.hasplural():
-                pluralform = gpo.po_message_msgid_plural(self._gpo_message)
-                if isinstance(pluralform, str):
-                    pluralform = pluralform.decode(self._encoding)
+                multi = multistring(singular, self._encoding)
+                pluralform = gpo.po_message_msgid_plural(self._gpo_message).decode(self._encoding)
                 multi.strings.append(pluralform)
-            return multi
+                return multi
+            else:
+                return singular
         else:
             return u""
 
@@ -225,9 +223,9 @@ class pounit(pocommon.pounit):
         if isinstance(source, unicode):
             source = source.encode(self._encoding)
         if isinstance(source, list):
-            gpo.po_message_set_msgid(self._gpo_message, str(source[0]))
+            gpo.po_message_set_msgid(self._gpo_message, source[0].encode(self._encoding))
             if len(source) > 1:
-                gpo.po_message_set_msgid_plural(self._gpo_message, str(source[1]))
+                gpo.po_message_set_msgid_plural(self._gpo_message, source[1].encode(self._encoding))
         else:
             gpo.po_message_set_msgid(self._gpo_message, source)
             gpo.po_message_set_msgid_plural(self._gpo_message, None)
@@ -240,7 +238,7 @@ class pounit(pocommon.pounit):
             nplural = 0
             plural = gpo.po_message_msgstr_plural(self._gpo_message, nplural)
             while plural:
-                plurals.append(plural)
+                plurals.append(plural.decode(self._encoding))
                 nplural += 1
                 plural = gpo.po_message_msgstr_plural(self._gpo_message, nplural)
             if plurals:
@@ -248,7 +246,7 @@ class pounit(pocommon.pounit):
             else:
                 multi = multistring(u"")
         else:
-            multi = multistring(gpo.po_message_msgstr(self._gpo_message) or u"", encoding=self._encoding)
+            multi = (gpo.po_message_msgstr(self._gpo_message) or "").decode(self._encoding)
         return multi
 
     def settarget(self, target):
@@ -259,7 +257,7 @@ class pounit(pocommon.pounit):
             elif isinstance(target, basestring):
                 target = [target]
         # for non-plurals: check number of items in 'target'
-        elif isinstance(target,(dict, list)):
+        elif isinstance(target, (dict, list)):
             if len(target) == 1:
                 target = target[0]
             else:
@@ -300,7 +298,7 @@ class pounit(pocommon.pounit):
     def getid(self):
         """The unique identifier for this unit according to the convensions in
         .mo files."""
-        id = gpo.po_message_msgid(self._gpo_message)
+        id = (gpo.po_message_msgid(self._gpo_message) or "").decode(self._encoding)
         # Gettext does not consider the plural to determine duplicates, only 
         # the msgid. For generation of .mo files, we might want to use this
         # code to generate the entry for the hash table, but for now, it is 
@@ -310,8 +308,8 @@ class pounit(pocommon.pounit):
 #            id = '%s\0%s' % (id, plural)
         context = gpo.po_message_msgctxt(self._gpo_message)
         if context:
-            id = "%s\04%s" % (context, id)
-        return id or ""
+            id = u"%s\04%s" % (context.decode(self._encoding), id)
+        return id
 
     def getnotes(self, origin=None):
         if origin == None:
@@ -361,7 +359,7 @@ class pounit(pocommon.pounit):
                     newlines.append(" " + line)
                 else:
                     newlines.append(line)
-            newnotes = "\n".join(newlines)
+            newnotes = "\n".join(newlines).encode(self._encoding)
             if origin in ["programmer", "developer", "source code"]:
                 gpo.po_message_set_extracted_comments(self._gpo_message, newnotes)
             else:
@@ -380,7 +378,6 @@ class pounit(pocommon.pounit):
 
         Overwrite non-blank self.msgstr only if overwrite is True
         merge comments only if comments is True
-        
         """
 
         if not isinstance(otherpo, pounit):
@@ -400,7 +397,7 @@ class pounit(pocommon.pounit):
             if self._extract_msgidcomments(otherpo.target):
                 otherpo.target = otherpo.target.replace('_: ' + otherpo._extract_msgidcomments()+ '\n', '')
             self.target = otherpo.target
-            if self.source != otherpo.source:
+            if self.source != otherpo.source or self.getcontext() != otherpo.getcontext():
                 self.markfuzzy()
             else:
                 self.markfuzzy(otherpo.isfuzzy())
@@ -417,11 +414,14 @@ class pounit(pocommon.pounit):
         return self.getid() == "" and len(self.target) > 0
 
     def isblank(self):
-        return len(self.source) == 0 and len(self.target) == 0
+        return len(self.source) == len(self.target) == len(self.getcontext()) == 0
 
     def hastypecomment(self, typecomment):
         return gpo.po_message_is_format(self._gpo_message, typecomment)
 
+    def settypecomment(self, typecomment, present=True):
+        gpo.po_message_set_format(self._gpo_message, typecomment, present)
+
     def hasmarkedcomment(self, commentmarker):
         commentmarker = "(%s)" % commentmarker
         for comment in self.getnotes("translator").split("\n"):
@@ -429,21 +429,12 @@ class pounit(pocommon.pounit):
                 return True
         return False
 
-    def istranslated(self):
-        return super(pounit, self).istranslated() and not self.isobsolete()
-
-    def istranslatable(self):
-        return not (self.isheader() or self.isblank() or self.isobsolete())
-
     def isfuzzy(self):
         return gpo.po_message_is_fuzzy(self._gpo_message)
 
     def markfuzzy(self, present=True):
         gpo.po_message_set_fuzzy(self._gpo_message, present)
 
-    def isreview(self):
-        return self.hasmarkedcomment("review") or self.hasmarkedcomment("pofilter")
-
     def isobsolete(self):
         return gpo.po_message_is_obsolete(self._gpo_message)
 
@@ -460,20 +451,21 @@ class pounit(pocommon.pounit):
 
     def _extract_msgidcomments(self, text=None):
         """Extract KDE style msgid comments from the unit.
-        
+
         @rtype: String
         @return: Returns the extracted msgidcomments found in this unit's msgid.
-        
         """
-    
         if not text:
-            text = gpo.po_message_msgid(self._gpo_message)
+            text = gpo.po_message_msgid(self._gpo_message).decode(self._encoding)
         if text:
-            msgidcomment = re.search("_: (.*)\n", text)
-            if msgidcomment:
-                return msgidcomment.group(1).decode(self._encoding)
+            return pocommon.extract_msgid_comment(text)
         return u""
 
+    def setmsgidcomment(self, msgidcomment):
+        if msgidcomment:
+            self.source = u"_: %s\n%s" % (msgidcomment, self.source)
+    msgidcomment = property(_extract_msgidcomments, setmsgidcomment)
+
     def __str__(self):
         pf = pofile()
         pf.addunit(self)
@@ -500,19 +492,55 @@ class pounit(pocommon.pounit):
             parts = loc.split(":")
             file = parts[0]
             if len(parts) == 2:
-                line = int(parts[1])
+                line = int(parts[1] or "0")
             else:
                 line = -1
             gpo.po_message_add_filepos(self._gpo_message, file, line)
 
     def getcontext(self):
         msgctxt = gpo.po_message_msgctxt(self._gpo_message)
-        msgidcomment = self._extract_msgidcomments()
         if msgctxt:
-            return msgctxt + msgidcomment
+            return msgctxt.decode(self._encoding)
         else:
+            msgidcomment = self._extract_msgidcomments()
             return msgidcomment
 
+    def buildfromunit(cls, unit):
+        """Build a native unit from a foreign unit, preserving as much
+        information as possible."""
+        if type(unit) == cls and hasattr(unit, "copy") and callable(unit.copy):
+            return unit.copy()
+        elif isinstance(unit, pocommon.pounit):
+            newunit = cls(unit.source)
+            newunit.target = unit.target
+            #context
+            newunit.msgidcomment = unit._extract_msgidcomments()
+            context = unit.getcontext()
+            if not newunit.msgidcomment and context:
+                gpo.po_message_set_msgctxt(newunit._gpo_message, context)
+
+            locations = unit.getlocations()
+            if locations:
+                newunit.addlocations(locations)
+            notes = unit.getnotes("developer")
+            if notes:
+                newunit.addnote(notes, "developer")
+            notes = unit.getnotes("translator")
+            if notes:
+                newunit.addnote(notes, "translator")
+            if unit.isobsolete():
+                newunit.makeobsolete()
+            newunit.markfuzzy(unit.isfuzzy())
+            for tc in ['python-format', 'c-format', 'php-format']:
+                if unit.hastypecomment(tc):
+                    newunit.settypecomment(tc)
+                    # We assume/guess/hope that there will only be one
+                    break
+            return newunit
+        else:
+            return base.TranslationUnit.buildfromunit(unit)
+    buildfromunit = classmethod(buildfromunit)
+
 class pofile(pocommon.pofile):
     UnitClass = pounit
     def __init__(self, inputfile=None, encoding=None, unitclass=pounit):
@@ -534,51 +562,42 @@ class pofile(pocommon.pofile):
 
     def removeduplicates(self, duplicatestyle="merge"):
         """make sure each msgid is unique ; merge comments etc from duplicates into original"""
-        msgiddict = {}
+        # TODO: can we handle consecutive calls to removeduplicates()? What
+        # about files already containing msgctxt? - test
+        id_dict = {}
         uniqueunits = []
-        # we sometimes need to keep track of what has been marked
-        # TODO: this is using a list as the pos aren't hashable, but this is slow...
+        # TODO: this is using a list as the pos aren't hashable, but this is slow.
+        # probably not used frequently enough to worry about it, though.
         markedpos = []
         def addcomment(thepo):
             thepo.msgidcomment = " ".join(thepo.getlocations())
             markedpos.append(thepo)
         for thepo in self.units:
-            if thepo.isheader():
+            id = thepo.getid()
+            if thepo.isheader() and not thepo.getlocations():
+                # header msgids shouldn't be merged...
                 uniqueunits.append(thepo)
-                continue
-            if duplicatestyle.startswith("msgid_comment"):
-                msgid = thepo._extract_msgidcomments() + thepo.source
-            else:
-                msgid = thepo.source
-            if duplicatestyle == "msgid_comment_all":
-                addcomment(thepo)
-                uniqueunits.append(thepo)
-            elif msgid in msgiddict:
+            elif id in id_dict:
                 if duplicatestyle == "merge":
-                    if msgid:
-                        msgiddict[msgid].merge(thepo)
+                    if id:
+                        id_dict[id].merge(thepo)
                     else:
                         addcomment(thepo)
                         uniqueunits.append(thepo)
-                elif duplicatestyle == "keep":
-                    uniqueunits.append(thepo)
-                elif duplicatestyle == "msgid_comment":
-                    origpo = msgiddict[msgid]
-                    if origpo not in markedpos:
-                        addcomment(origpo)
-                    addcomment(thepo)
-                    uniqueunits.append(thepo)
                 elif duplicatestyle == "msgctxt":
-                    origpo = msgiddict[msgid]
+                    origpo = id_dict[id]
                     if origpo not in markedpos:
                         gpo.po_message_set_msgctxt(origpo._gpo_message, " ".join(origpo.getlocations()))
                         markedpos.append(thepo)
                     gpo.po_message_set_msgctxt(thepo._gpo_message, " ".join(thepo.getlocations()))
                     uniqueunits.append(thepo)
             else:
-                if not msgid and duplicatestyle != "keep":
-                    addcomment(thepo)
-                msgiddict[msgid] = thepo
+                if not id:
+                    if duplicatestyle == "merge":
+                        addcomment(thepo)
+                    else:
+                        gpo.po_message_set_msgctxt(thepo._gpo_message, " ".join(thepo.getlocations()))
+                id_dict[id] = thepo
                 uniqueunits.append(thepo)
         new_gpo_memory_file = gpo.po_file_create()
         new_gpo_message_iterator = gpo.po_message_iterator(new_gpo_memory_file, None)
@@ -604,11 +623,13 @@ class pofile(pocommon.pofile):
         outputstring = ""
         if self._gpo_memory_file:
             obsolete_workaround()
-            f = tempfile.NamedTemporaryFile(prefix='translate', suffix='.po')
-            self._gpo_memory_file = gpo.po_file_write_v2(self._gpo_memory_file, f.name, xerror_handler)
-            f.seek(0)
+            f, fname = tempfile.mkstemp(prefix='translate', suffix='.po')
+            os.close(f)
+            self._gpo_memory_file = gpo.po_file_write_v2(self._gpo_memory_file, fname, xerror_handler)
+            f = open(fname)
             outputstring = f.read()
             f.close()
+            os.remove(fname)
         return outputstring
 
     def isempty(self):
@@ -662,7 +683,7 @@ class pofile(pocommon.pofile):
         self._gpo_message_iterator = gpo.po_message_iterator(self._gpo_memory_file, None)
         newmessage = gpo.po_next_message(self._gpo_message_iterator)
         while newmessage:
-            newunit = pounit(gpo_message=newmessage)
+            newunit = pounit(gpo_message=newmessage, encoding=self._encoding)
             self.addunit(newunit, new=False)
             newmessage = gpo.po_next_message(self._gpo_message_iterator)
         self._free_iterator()
