@@ -1,42 +1,35 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright 2006-2007 Zuza Software Foundation
-# 
-# This file is part of translate.
+# Copyright 2006-2009 Zuza Software Foundation
 #
-# translate is free software; you can redistribute it and/or modify
+# This file is part of the Translate Toolkit.
+#
+# This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
-# translate is distributed in the hope that it will be useful,
+#
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-#
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 """Parent class for LISA standards (TMX, TBX, XLIFF)"""
 
 import re
 
 from translate.storage import base
-from translate.storage.placeables import lisa
 from translate.lang import data
 try:
     from lxml import etree
+    from translate.misc.xml_helpers import *
 except ImportError, e:
     raise ImportError("lxml is not installed. It might be possible to continue without support for XML formats.")
 
-string_xpath = etree.XPath("string()")
-
-def getText(node):
-    """joins together the text from all the text nodes in the nodelist and their children"""
-    return unicode(string_xpath(node)) # specific to lxml.etree
 
 def _findAllMatches(text, re_obj):
     """generate match objects for all L{re_obj} matches in L{text}."""
@@ -44,10 +37,12 @@ def _findAllMatches(text, re_obj):
     max = len(text)
     while start < max:
         m = re_obj.search(text, start)
-        if not m: break
+        if not m:
+            break
         yield m
         start = m.end()
 
+#TODO: we can now do better with our proper placeables support
 placeholders = ['(%[diouxXeEfFgGcrs])', r'(\\+.?)', '(%[0-9]$lx)', '(%[0-9]\$[a-z])', '(<.+?>)']
 re_placeholders = [re.compile(ph) for ph in placeholders]
 def _getPhMatches(text):
@@ -60,35 +55,12 @@ def _getPhMatches(text):
     matches.sort(lambda a, b: cmp(a.start(), b.start()))
     return matches
 
-XML_NS = 'http://www.w3.org/XML/1998/namespace'
-
-def getXMLlang(node):
-    """Sets the xml:lang attribute on node"""
-    return node.get("{%s}lang" % XML_NS)
-
-def setXMLlang(node, lang):
-    """Sets the xml:lang attribute on node"""
-    node.set("{%s}lang" % XML_NS, lang)
-
-def setXMLspace(node, value):
-    """Sets the xml:space attribute on node"""
-    node.set("{%s}space" % XML_NS, value)
-
-def namespaced(namespace, name):
-    """Returns name in Clark notation within the given namespace.
-
-       For example namespaced("source") in an XLIFF document might return::
-           {urn:oasis:names:tc:xliff:document:1.1}source
-       This is needed throughout lxml.
-    """
-    if namespace:
-        return "{%s}%s" % (namespace, name)
-    else:
-        return name
 
 class LISAunit(base.TranslationUnit):
-    """A single unit in the file. 
-Provisional work is done to make several languages possible."""
+    """
+    A single unit in the file.  Provisional work is done to make several
+    languages possible.
+    """
 
     #The name of the root element of this unit type:(termEntry, tu, trans-unit)
     rootNode = ""
@@ -98,10 +70,16 @@ Provisional work is done to make several languages possible."""
     textNode = ""
 
     namespace = None
+    _default_xml_space = "preserve"
+    """The default handling of spacing in the absense of an xml:space attribute.
+
+    This is mostly for correcting XLIFF behaviour."""
 
     def __init__(self, source, empty=False, **kwargs):
         """Constructs a unit containing the given source string"""
         if empty:
+            self._rich_source = None
+            self._rich_target = None
             return
         self.xmlelement = etree.Element(self.rootNode)
         #add descrip, note, etc.
@@ -109,13 +87,15 @@ Provisional work is done to make several languages possible."""
 
     def __eq__(self, other):
         """Compares two units"""
+        if not isinstance(other, LISAunit):
+            return super(LISAunit, self).__eq__(other)
         languageNodes = self.getlanguageNodes()
         otherlanguageNodes = other.getlanguageNodes()
         if len(languageNodes) != len(otherlanguageNodes):
             return False
         for i in range(len(languageNodes)):
-            mytext = self.getNodeText(languageNodes[i])
-            othertext = other.getNodeText(otherlanguageNodes[i])
+            mytext = self.getNodeText(languageNodes[i], getXMLspace(self.xmlelement, self._default_xml_space))
+            othertext = other.getNodeText(otherlanguageNodes[i], getXMLspace(self.xmlelement, self._default_xml_space))
             if mytext != othertext:
                 #TODO:^ maybe we want to take children and notes into account
                 return False
@@ -136,38 +116,24 @@ Provisional work is done to make several languages possible."""
             self.xmlelement[0] = dom_node
         else:
             self.xmlelement.append(dom_node)
-    
+
     def get_source_dom(self):
         return self.getlanguageNode(lang=None, index=0)
     source_dom = property(get_source_dom, set_source_dom)
 
-    def _ensure_singular(cls, value):
-        if value is not None and len(value) > 1:
-            raise Exception("XLIFF cannot handle plurals by default")
-    _ensure_singular = classmethod(_ensure_singular)
-
-    def set_rich_source(self, value, sourcelang='en'):
-        self._ensure_singular(value)
-        sourcelanguageNode = self.createlanguageNode(sourcelang, u'', "source")        
-        self.source_dom = lisa.insert_into_dom(sourcelanguageNode, value[0])
-
-    def get_rich_source(self):
-        return [lisa.extract_chunks(self.source_dom)]
-    rich_source = property(get_rich_source, set_rich_source)
-
     def setsource(self, text, sourcelang='en'):
         text = data.forceunicode(text)
         self.source_dom = self.createlanguageNode(sourcelang, text, "source")
 
     def getsource(self):
-        return self.getNodeText(self.source_dom)
+        return self.getNodeText(self.source_dom, getXMLspace(self.xmlelement, self._default_xml_space))
     source = property(getsource, setsource)
 
     def set_target_dom(self, dom_node, append=False):
         languageNodes = self.getlanguageNodes()
         assert len(languageNodes) > 0
         if dom_node is not None:
-            if append or len(languageNodes) == 1:
+            if append or len(languageNodes) == 0:
                 self.xmlelement.append(dom_node)
             else:
                 self.xmlelement.insert(1, dom_node)
@@ -181,20 +147,6 @@ Provisional work is done to make several languages possible."""
             return self.getlanguageNode(lang=None, index=1)
     target_dom = property(get_target_dom)
 
-    def set_rich_target(self, value, lang='xx', append=False):
-        self._ensure_singular(value)
-        languageNode = None
-        if not value is None:
-            languageNode = self.createlanguageNode(lang, u'', "target")
-            lisa.insert_into_dom(languageNode, value[0])
-        self.set_target_dom(languageNode, append)
-
-    def get_rich_target(self, lang=None):
-        """retrieves the "target" text (second entry), or the entry in the 
-        specified language, if it exists"""
-        return [lisa.extract_chunks(self.get_target_dom(lang))]
-    rich_target = property(get_rich_target, set_rich_target)
-
     def settarget(self, text, lang='xx', append=False):
         #XXX: we really need the language - can't really be optional, and we
         # need to propagate it
@@ -220,13 +172,13 @@ Provisional work is done to make several languages possible."""
             self.set_target_dom(None, False)
 
     def gettarget(self, lang=None):
-        """retrieves the "target" text (second entry), or the entry in the 
+        """retrieves the "target" text (second entry), or the entry in the
         specified language, if it exists"""
-        return self.getNodeText(self.get_target_dom(lang))
+        return self.getNodeText(self.get_target_dom(lang), getXMLspace(self.xmlelement, self._default_xml_space))
     target = property(gettarget, settarget)
 
     def createlanguageNode(self, lang, text, purpose=None):
-        """Returns a xml Element setup with given parameters to represent a 
+        """Returns a xml Element setup with given parameters to represent a
         single language entry. Has to be overridden."""
         return None
 
@@ -279,7 +231,7 @@ Provisional work is done to make several languages possible."""
                 return languageNodes[index]
         return None
 
-    def getNodeText(self, languageNode):
+    def getNodeText(self, languageNode, xml_space="preserve"):
         """Retrieves the term from the given languageNode"""
         if languageNode is None:
             return None
@@ -288,9 +240,9 @@ Provisional work is done to make several languages possible."""
             if terms is None:
                 return None
             else:
-                return getText(terms.next())
+                return getText(terms.next(), xml_space)
         else:
-            return getText(languageNode)
+            return getText(languageNode, xml_space)
 
     def __str__(self):
         return etree.tostring(self.xmlelement, pretty_print=True, encoding='utf-8')
@@ -324,8 +276,6 @@ class LISAfile(base.TranslationStore):
 
     def __init__(self, inputfile=None, sourcelanguage='en', targetlanguage=None, unitclass=None):
         super(LISAfile, self).__init__(unitclass=unitclass)
-        self.setsourcelanguage(sourcelanguage)
-        self.settargetlanguage(targetlanguage)
         if inputfile is not None:
             self.parse(inputfile)
             assert self.document.getroot().tag == self.namespaced(self.rootNode)
@@ -333,6 +283,8 @@ class LISAfile(base.TranslationStore):
             # We strip out newlines to ensure that spaces in the skeleton doesn't
             # interfere with the the pretty printing of lxml
             self.parse(self.XMLskeleton.replace("\n", ""))
+            self.setsourcelanguage(sourcelanguage)
+            self.settargetlanguage(targetlanguage)
             self.addheader()
         self._encoding = "UTF-8"
 
@@ -379,8 +331,8 @@ class LISAfile(base.TranslationStore):
             xml.seek(0)
             posrc = xml.read()
             xml = posrc
-        if etree.LXML_VERSION > (2, 1, 0):
-            #Since version 2.1.0 we can pass the strip_cdata parameter to 
+        if etree.LXML_VERSION >= (2, 1, 0):
+            #Since version 2.1.0 we can pass the strip_cdata parameter to
             #indicate that we don't want cdata to be converted to raw XML
             parser = etree.XMLParser(strip_cdata=False)
         else:
