@@ -20,22 +20,24 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 """A class loader that will load C or Python implementations of the PO class
-depending on the use_implementation variable
+depending on the USECPO variable.
 
-Use the environment variable USECPO to choose the C implementation which
+Use the environment variable USECPO=2 (or 1) to choose the C implementation which
 uses Gettext's libgettextpo for high parsing speed.  Otherise the local 
-Python based parser is used (slower but very well tested)"""
+Python based parser is used (slower but very well tested)."""
 
 import os
-import sys
+import logging
 
 if os.getenv('USECPO'):
     if os.getenv('USECPO') == "1":
-        print >> sys.stderr, "Using cPO"
+        logging.info("Using cPO")
         from cpo import *
+    elif os.getenv('USECPO') == "2":
+        logging.info("Using new cPO")
+        from fpo import *
     else:
-        print >> sys.stderr, "Using Python PO"
+        logging.info("Using Python PO")
         from pypo import *
 else:
     from pypo import *
-
