@@ -1,31 +1,31 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright 2006-2007 Zuza Software Foundation
-# 
-# This file is part of translate.
+# Copyright 2006-2009 Zuza Software Foundation
 #
-# translate is free software; you can redistribute it and/or modify
+# This file is part of the Translate Toolkit.
+#
+# This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
-# translate is distributed in the hope that it will be useful,
+#
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-#
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-"""An xliff file specifically suited for handling the po representation of 
-xliff. """
+"""XLIFF classes specifically suited for handling the PO representation in
+XLIFF.
 
-from translate.storage import xliff
-from translate.storage import lisa 
-from translate.storage import poheader
+This way the API supports plurals as if it was a PO file, for example.
+"""
+
+from translate.storage import base, lisa, poheader, xliff
+from translate.storage.placeables import general
 from translate.misc.multistring import multistring
 from lxml import etree
 import re
@@ -37,9 +37,14 @@ def hasplurals(thing):
 
 class PoXliffUnit(xliff.xliffunit):
     """A class to specifically handle the plural units created from a po file."""
+
+    rich_parsers = general.parsers
+
     def __init__(self, source=None, empty=False, encoding="UTF-8"):
+        self._rich_source = None
+        self._rich_target = None
         self.units = []
-            
+
         if empty:
             return
 
@@ -55,8 +60,6 @@ class PoXliffUnit(xliff.xliffunit):
         if isinstance(other, PoXliffUnit):
             if len(self.units) != len(other.units):
                 return False
-            if len(self.units) == 0:
-                return True
             if not super(PoXliffUnit, self).__eq__(other):
                 return False
             for i in range(len(self.units)-1):
@@ -70,8 +73,16 @@ class PoXliffUnit(xliff.xliffunit):
                 return self.source == other.source and self.target == other.target
         return False
 
+#XXX: We don't return language nodes correctly at the moment
+#    def getlanguageNodes(self):
+#        if not self.hasplural():
+#            return super(PoXliffUnit, self).getlanguageNodes()
+#        else:
+#            return self.units[0].getlanguageNodes()
+
     def setsource(self, source, sourcelang="en"):
 #        TODO: consider changing from plural to singular, etc.
+        self._rich_source = None
         if not hasplurals(source):
             super(PoXliffUnit, self).setsource(source, sourcelang)
         else:
@@ -89,13 +100,24 @@ class PoXliffUnit(xliff.xliffunit):
                 self.xmlelement.append(newunit.xmlelement)
             self.target = target
 
+    # We don't support any rich strings yet
+    multistring_to_rich = base.TranslationUnit.multistring_to_rich
+    rich_to_multistring = base.TranslationUnit.rich_to_multistring
+
+    rich_source = base.TranslationUnit.rich_source
+    rich_target = base.TranslationUnit.rich_target
+
     def getsource(self):
-        strings = [super(PoXliffUnit, self).getsource()]
-        strings.extend([unit.source for unit in self.units[1:]])
-        return multistring(strings)
+        if not self.hasplural():
+            return super(PoXliffUnit, self).getsource()
+        else:
+            strings = []
+            strings.extend([unit.source for unit in self.units])
+            return multistring(strings)
     source = property(getsource, setsource)
-    
+
     def settarget(self, text, lang='xx', append=False):
+        self._rich_target = None
         if self.gettarget() == text:
             return
         if not self.hasplural():
@@ -116,7 +138,7 @@ class PoXliffUnit(xliff.xliffunit):
             targets = text.strings + [""] * (sourcel - targetl)
         else:
             targets = text.strings
-        
+
         for i in range(len(self.units)):
             self.units[i].target = targets[i]
 
@@ -213,8 +235,8 @@ class PoXliffUnit(xliff.xliffunit):
             for (type, text) in commentpairs:
                 comments.append(text)
         return "\n".join(comments)
-    
-    def gettranslatorcomments(self):       
+
+    def gettranslatorcomments(self):
         """Returns the translator comments (x-po-trancomment), which corresponds
         to the # style po comments."""
         def hastrancomment((type, text)):
@@ -308,7 +330,7 @@ class PoXliffFile(xliff.xlifffile, poheader.poheader):
                 pluralnum += 1
                 group.append(unit.xmlelement)
                 self.units.append(unit)
-        
+
         return self.units[-pluralnum]
 
     def parse(self, xml):
@@ -317,10 +339,10 @@ class PoXliffFile(xliff.xlifffile, poheader.poheader):
         def ispluralgroup(node):
             """determines whether the xml node refers to a getttext plural"""
             return node.get("restype") == "x-gettext-plurals"
-        
+
         def isnonpluralunit(node):
             """determindes whether the xml node contains a plural like id.
-            
+
             We want to filter out all the plural nodes, except the very first
             one in each group.
             """
@@ -329,7 +351,7 @@ class PoXliffFile(xliff.xlifffile, poheader.poheader):
         def pluralunits(pluralgroups):
             for pluralgroup in pluralgroups:
                 yield self.UnitClass.createfromxmlElement(pluralgroup, namespace=self.namespace)
-        
+
         self.filename = getattr(xml, 'name', '')
         if hasattr(xml, "read"):
             xml.seek(0)
@@ -354,7 +376,7 @@ class PoXliffFile(xliff.xlifffile, poheader.poheader):
 
         for entry in singularunits:
             term = self.UnitClass.createfromxmlElement(entry, namespace=self.namespace)
-            if nextplural and unicode(term.source) in nextplural.source.strings:
+            if nextplural and unicode(term.getid()) == ("%s[0]" % nextplural.getid()):
                 self.addunit(nextplural, new=False)
                 try:
                     nextplural = pluralunit_iter.next()
