@@ -1,23 +1,22 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# 
-# Copyright 2002-2007 Zuza Software Foundation
-# 
-# This file is part of translate.
 #
-# translate is free software; you can redistribute it and/or modify
+# Copyright 2002-2009 Zuza Software Foundation
+#
+# This file is part of the Translate Toolkit.
+#
+# This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
-# translate is distributed in the hope that it will be useful,
+#
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 """classes that hold units of .po files (pounit) or entire files (pofile)
 gettext-style .po (or .pot) files are used in translations for KDE et al (see kbabel)"""
@@ -43,7 +42,7 @@ po_escape_map = dict([(value, key) for (key, value) in po_unescape_map.items()])
 
 def escapeforpo(line):
     """Escapes a line for po format. assumes no \n occurs in the line.
-    
+
     @param line: unescaped text
     """
     special_locations = []
@@ -104,7 +103,7 @@ def quoteforpo(text):
 
 def extractpoline(line):
     """Remove quote and unescape line from po file.
-     
+
     @param line: a quoted line from a po file (msgid or msgstr)
     """
     extracted = quote.extractwithoutquotes(line, '"', '"', '\\', includeescapes=unescapehandler)[0]
@@ -116,7 +115,8 @@ def unquotefrompo(postr):
 def encodingToUse(encoding):
     """Tests whether the given encoding is known in the python runtime, or returns utf-8.
     This function is used to ensure that a valid encoding is always used."""
-    if encoding == "CHARSET" or encoding == None: return 'utf-8'
+    if encoding == "CHARSET" or encoding == None:
+        return 'utf-8'
     return encoding
 #    if encoding is None: return False
 #    return True
@@ -150,6 +150,10 @@ class pounit(pocommon.pounit):
     # msgid = []
     # msgstr = []
 
+    # Our homegrown way to indicate what must be copied in a shallow
+    # fashion
+    __shallow__ = ['_store']
+
     def __init__(self, source=None, encoding="UTF-8"):
         self._encoding = encodingToUse(encoding)
         self.obsolete = False
@@ -180,10 +184,10 @@ class pounit(pocommon.pounit):
             self.obsoletemsgidcomments = []
 
     def _get_all_comments(self):
-        return [self.othercomments, 
-                self.automaticcomments, 
-                self.sourcecomments, 
-                self.typecomments, 
+        return [self.othercomments,
+                self.automaticcomments,
+                self.sourcecomments,
+                self.typecomments,
                 self.msgidcomments,
                 self.obsoletemsgidcomments]
 
@@ -222,7 +226,7 @@ class pounit(pocommon.pounit):
 
     def setsource(self, source):
         """Sets the msgid to the given (unescaped) value.
-        
+
         @param source: an unescaped source string.
         """
         self.msgid, self.msgid_plural = self._set_source_vars(source)
@@ -234,7 +238,7 @@ class pounit(pocommon.pounit):
 
     def _set_prev_source(self, source):
         """Sets the msgid to the given (unescaped) value.
-        
+
         @param source: an unescaped source string.
         """
         self.prev_msgid, self.prev_msgid_plural = self._set_source_vars(source)
@@ -250,6 +254,7 @@ class pounit(pocommon.pounit):
 
     def settarget(self, target):
         """Sets the msgstr to the given (unescaped) value"""
+        self._rich_target = None
         if isinstance(target, str):
             target = target.decode(self._encoding)
         if self.hasplural():
@@ -257,7 +262,7 @@ class pounit(pocommon.pounit):
                 target = target.strings
             elif isinstance(target, basestring):
                 target = [target]
-        elif isinstance(target,(dict, list)):
+        elif isinstance(target, (dict, list)):
             if len(target) == 1:
                 target = target[0]
             else:
@@ -295,6 +300,7 @@ class pounit(pocommon.pounit):
         text = data.forceunicode(text)
         commentlist = self.othercomments
         linestart = "# "
+        autocomments = False
         if origin in ["programmer", "developer", "source code"]:
             autocomments = True
             commentlist = self.automaticcomments
@@ -309,11 +315,30 @@ class pounit(pocommon.pounit):
                 self.automaticcomments = newcomments
             else:
                 self.othercomments = newcomments
-        
+
     def removenotes(self):
         """Remove all the translator's notes (other comments)"""
         self.othercomments = []
 
+    def __deepcopy__(self, memo={}):
+        # Make an instance to serve as the copy
+        new_unit = self.__class__()
+        # We'll be testing membership frequently, so make a set from
+        # self.__shallow__
+        shallow = set(self.__shallow__)
+        # Make deep copies of all members which are not in shallow
+        for key, value in self.__dict__.iteritems():
+            if key not in shallow:
+                setattr(new_unit, key, copy.deepcopy(value))
+        # Make shallow copies of all members which are in shallow
+        for key in set(shallow):
+            setattr(new_unit, key, getattr(self, key))
+        # Mark memo with ourself, so that we won't get deep copied
+        # again
+        memo[id(self)] = self
+        # Return our copied unit
+        return new_unit
+
     def copy(self):
         return copy.deepcopy(self)
 
@@ -335,7 +360,6 @@ class pounit(pocommon.pounit):
 
         Overwrite non-blank self.msgstr only if overwrite is True
         merge comments only if comments is True
-        
         """
 
         def mergelists(list1, list2, split=False):
@@ -347,7 +371,7 @@ class pounit(pocommon.pounit):
                 for position, item in enumerate(list2):
                     if isinstance(item, str):
                         list2[position] = item.decode("utf-8")
-                        
+
             #Determine the newline style of list1
             lineend = ""
             if list1 and list1[0]:
@@ -358,7 +382,7 @@ class pounit(pocommon.pounit):
                     lineend = ""
             else:
                 lineend = "\n"
-            
+
             #Split if directed to do so:
             if split:
                 splitlist1 = []
@@ -397,7 +421,7 @@ class pounit(pocommon.pounit):
             if self._extract_msgidcomments(otherpo.target):
                 otherpo.target = otherpo.target.replace('_: ' + otherpo._extract_msgidcomments()+ '\n', '')
             self.target = otherpo.target
-            if self.source != otherpo.source:
+            if self.source != otherpo.source or self.getcontext() != otherpo.getcontext():
                 self.markfuzzy()
             else:
                 self.markfuzzy(otherpo.isfuzzy())
@@ -410,9 +434,9 @@ class pounit(pocommon.pounit):
 
     def isheader(self):
         #return (self._msgidlen() == 0) and (self._msgstrlen() > 0) and (len(self.msgidcomments) == 0)
-        #rewritten here for performance:        
-        return (is_null(self.msgid) 
-                        and not is_null(self.msgstr) 
+        #rewritten here for performance:
+        return (is_null(self.msgid)
+                        and not is_null(self.msgstr)
                         and self.msgidcomments == []
                         and is_null(self.msgctxt)
         )
@@ -420,7 +444,7 @@ class pounit(pocommon.pounit):
     def isblank(self):
         if self.isheader() or len(self.msgidcomments):
             return False
-        if (self._msgidlen() == 0) and (self._msgstrlen() == 0):
+        if (self._msgidlen() == 0) and (self._msgstrlen() == 0) and (is_null(self.msgctxt)):
             return True
         return False
         # TODO: remove:
@@ -428,12 +452,12 @@ class pounit(pocommon.pounit):
         # return len(self.source.strip()) == 0
 
     def hastypecomment(self, typecomment):
-        """check whether the given type comment is present"""
+        """Check whether the given type comment is present"""
         # check for word boundaries properly by using a regular expression...
         return sum(map(lambda tcline: len(re.findall("\\b%s\\b" % typecomment, tcline)), self.typecomments)) != 0
 
     def hasmarkedcomment(self, commentmarker):
-        """check whether the given comment marker is present as # (commentmarker) ..."""
+        """Check whether the given comment marker is present as # (commentmarker) ..."""
         commentmarker = "(%s)" % commentmarker
         for comment in self.othercomments:
             if comment.replace("#", "", 1).strip().startswith(commentmarker):
@@ -441,7 +465,7 @@ class pounit(pocommon.pounit):
         return False
 
     def settypecomment(self, typecomment, present=True):
-        """alters whether a given typecomment is present"""
+        """Alters whether a given typecomment is present"""
         if self.hastypecomment(typecomment) != present:
             if present:
                 self.typecomments.append("#, %s\n" % typecomment)
@@ -450,21 +474,12 @@ class pounit(pocommon.pounit):
                 typecomments = map(lambda tcline: re.sub("\\b%s\\b[ \t,]*" % typecomment, "", tcline), self.typecomments)
                 self.typecomments = filter(lambda tcline: tcline.strip() != "#,", typecomments)
 
-    def istranslated(self):
-        return super(pounit, self).istranslated() and not self.isobsolete()
-
-    def istranslatable(self):
-        return not (self.isheader() or self.isblank())
-
     def isfuzzy(self):
         return self.hastypecomment("fuzzy")
 
     def markfuzzy(self, present=True):
         self.settypecomment("fuzzy", present)
 
-    def isreview(self):
-        return self.hastypecomment("review") or self.hasmarkedcomment("review") or self.hasmarkedcomment("pofilter")
-
     def isobsolete(self):
         return self.obsolete
 
@@ -601,12 +616,12 @@ class pounit(pocommon.pounit):
         # if there's no msgid don't do msgid and string, unless we're the header
         # this will also discard any comments other than plain othercomments...
         if is_null(self.msgid):
-            if not (self.isheader() or self.msgidcomments or self.sourcecomments):
+            if not (self.isheader() or self.getcontext() or self.sourcecomments):
                 return "".join(lines)
         lines.extend(self.automaticcomments)
         lines.extend(self.sourcecomments)
-        add_prev_msgid_info(lines)
         lines.extend(self.typecomments)
+        add_prev_msgid_info(lines)
         if self.msgctxt:
             lines.append(self._getmsgpartstr("msgctxt", self.msgctxt))
         lines.append(self._getmsgpartstr("msgid", self.msgid, self.msgidcomments))
@@ -640,16 +655,23 @@ class pounit(pocommon.pounit):
 
     def _extract_msgidcomments(self, text=None):
         """Extract KDE style msgid comments from the unit.
-        
+
         @rtype: String
         @return: Returns the extracted msgidcomments found in this unit's msgid.
-        
         """
 
         if not text:
             text = unquotefrompo(self.msgidcomments)
         return text.split('\n')[0].replace('_: ', '', 1)
 
+    def setmsgidcomment(self, msgidcomment):
+        if msgidcomment:
+            self.msgidcomments = ['"_: %s\\n"' % msgidcomment]
+        else:
+            self.msgidcomments = []
+
+    msgidcomment = property(_extract_msgidcomments, setmsgidcomment)
+    
     def getcontext(self):
         """Get the message context."""
         return unquotefrompo(self.msgctxt) + self._extract_msgidcomments()
@@ -664,16 +686,17 @@ class pounit(pocommon.pounit):
 #        id = '\0'.join(self.source.strings)
         id = self.source
         if self.msgidcomments:
-            id = "_: %s\n%s" % (context, id)
+            id = u"_: %s\n%s" % (context, id)
         elif context:
-            id = "%s\04%s" % (context, id)
+            id = u"%s\04%s" % (context, id)
         return id
 
 class pofile(pocommon.pofile):
-    """this represents a .po file containing various units"""
+    """A .po file containing various units"""
     UnitClass = pounit
+
     def __init__(self, inputfile=None, encoding=None, unitclass=pounit):
-        """construct a pofile, optionally reading in from inputfile.
+        """Construct a pofile, optionally reading in from inputfile.
         encoding can be specified but otherwise will be read from the PO header"""
         self.UnitClass = unitclass
         pocommon.pofile.__init__(self, unitclass=unitclass)
@@ -684,7 +707,12 @@ class pofile(pocommon.pofile):
             self.parse(inputfile)
 
     def changeencoding(self, newencoding):
-        """changes the encoding on the file"""
+        """Deprecated: changes the encoding on the file."""
+        # This should not be here but in poheader. It also shouldn't mangle the
+        # header itself, but use poheader methods. All users are removed, so
+        # we can deprecate after one release.
+        raise DeprecationWarning
+
         self._encoding = encodingToUse(newencoding)
         if not self.units:
             return
@@ -694,9 +722,11 @@ class pofile(pocommon.pofile):
         charsetline = None
         headerstr = unquotefrompo(header.msgstr)
         for line in headerstr.split("\n"):
-            if not ":" in line: continue
+            if not ":" in line:
+                continue
             key, value = line.strip().split(":", 1)
-            if key.strip() != "Content-Type": continue
+            if key.strip() != "Content-Type":
+                continue
             charsetline = line
         if charsetline is None:
             headerstr += "Content-Type: text/plain; charset=%s" % self._encoding
@@ -714,7 +744,7 @@ class pofile(pocommon.pofile):
         header.msgstr = quoteforpo(headerstr)
 
     def parse(self, input):
-        """parses the given file or file source string"""
+        """Parses the given file or file source string."""
         try:
             if hasattr(input, 'name'):
                 self.filename = input.name
@@ -727,57 +757,48 @@ class pofile(pocommon.pofile):
             raise base.ParseError(e)
 
     def removeduplicates(self, duplicatestyle="merge"):
-        """make sure each msgid is unique ; merge comments etc from duplicates into original"""
-        msgiddict = {}
+        """Make sure each msgid is unique ; merge comments etc from duplicates into original"""
+        # TODO: can we handle consecutive calls to removeduplicates()? What
+        # about files already containing msgctxt? - test
+        id_dict = {}
         uniqueunits = []
-        # we sometimes need to keep track of what has been marked
-        # TODO: this is using a list as the pos aren't hashable, but this is slow...
+        # TODO: this is using a list as the pos aren't hashable, but this is slow.
+        # probably not used frequently enough to worry about it, though.
         markedpos = []
         def addcomment(thepo):
             thepo.msgidcomments.append('"_: %s\\n"' % " ".join(thepo.getlocations()))
             markedpos.append(thepo)
         for thepo in self.units:
-            if duplicatestyle.startswith("msgid_comment"):
-                msgid = unquotefrompo(thepo.msgidcomments) + unquotefrompo(thepo.msgid)
-            else:
-                msgid = unquotefrompo(thepo.msgid)
+            id = thepo.getid()
             if thepo.isheader() and not thepo.getlocations():
                 # header msgids shouldn't be merged...
                 uniqueunits.append(thepo)
-            elif duplicatestyle == "msgid_comment_all":
-                addcomment(thepo)
-                uniqueunits.append(thepo)
-            elif msgid in msgiddict:
+            elif id in id_dict:
                 if duplicatestyle == "merge":
-                    if msgid:
-                        msgiddict[msgid].merge(thepo)
+                    if id:
+                        id_dict[id].merge(thepo)
                     else:
                         addcomment(thepo)
                         uniqueunits.append(thepo)
-                elif duplicatestyle == "keep":
-                    uniqueunits.append(thepo)
-                elif duplicatestyle == "msgid_comment":
-                    origpo = msgiddict[msgid]
-                    if origpo not in markedpos:
-                        addcomment(origpo)
-                    addcomment(thepo)
-                    uniqueunits.append(thepo)
                 elif duplicatestyle == "msgctxt":
-                    origpo = msgiddict[msgid]
+                    origpo = id_dict[id]
                     if origpo not in markedpos:
-                        origpo.msgctxt.append('"%s"' % " ".join(origpo.getlocations()))
+                        origpo.msgctxt.append('"%s"' % escapeforpo(" ".join(origpo.getlocations())))
                         markedpos.append(thepo)
-                    thepo.msgctxt.append('"%s"' % " ".join(thepo.getlocations()))
+                    thepo.msgctxt.append('"%s"' % escapeforpo(" ".join(thepo.getlocations())))
                     uniqueunits.append(thepo)
             else:
-                if not msgid and duplicatestyle != "keep":
-                    addcomment(thepo)
-                msgiddict[msgid] = thepo
+                if not id:
+                    if duplicatestyle == "merge":
+                        addcomment(thepo)
+                    else:
+                        thepo.msgctxt.append('"%s"' % escapeforpo(" ".join(thepo.getlocations())))
+                id_dict[id] = thepo
                 uniqueunits.append(thepo)
         self.units = uniqueunits
 
     def __str__(self):
-        """convert to a string. double check that unicode is handled somehow here"""
+        """Convert to a string. double check that unicode is handled somehow here"""
         output = self._getoutput()
         if isinstance(output, unicode):
             return output.encode(getattr(self, "encoding", "UTF-8"))
@@ -791,7 +812,8 @@ class pofile(pocommon.pofile):
             lines.append(unitsrc)
         lines = "".join(self.encode(lines)).rstrip()
         #After the last pounit we will have \n\n and we only want to end in \n:
-        if lines: lines += "\n"
+        if lines:
+            lines += "\n"
         return lines
 
     def encode(self, lines):
@@ -822,4 +844,3 @@ class pofile(pocommon.pofile):
         for unit in self.units:
             if not (unit.isheader() or unit.isobsolete()):
                 yield unit
-
