@@ -1,24 +1,23 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright 2007 Zuza Software Foundation
+# Copyright 2007-2009 Zuza Software Foundation
 #
-# This file is part of translate.
+# This file is part of the Translate Toolkit.
 #
-# translate is free software; you can redistribute it and/or modify
+# This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 #
-# translate is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-from UserDict import UserDict
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
+
 
 """Module to provide a cache of statistics in a database.
 
@@ -27,8 +26,10 @@ from UserDict import UserDict
 @license: U{GPL <http://www.fsf.org/licensing/licenses/gpl.html>}
 """
 
+from UserDict import UserDict
+
 from translate import __version__ as toolkitversion
-from translate.storage import factory, base
+from translate.storage import factory
 from translate.misc.multistring import multistring
 from translate.lang.common import Common
 
@@ -40,6 +41,7 @@ import os.path
 import re
 import sys
 import stat
+import thread
 
 kdepluralre = re.compile("^_n: ")
 brtagre = re.compile("<br\s*?/?>")
@@ -110,11 +112,11 @@ class Record(UserDict):
 def transaction(f):
     """Modifies f to commit database changes if it executes without exceptions.
     Otherwise it rolls back the database.
-    
+
     ALL publicly accessible methods in StatsCache MUST be decorated with this
     decorator.
     """
-    
+
     def decorated_f(self, *args, **kwargs):
         try:
             result = f(self, *args, **kwargs)
@@ -124,7 +126,8 @@ def transaction(f):
             # If ANY exception is raised, we're left in an
             # uncertain state and we MUST roll back any changes to avoid getting
             # stuck in an inconsistent state.
-            self.con.rollback()
+            if self.con:
+                self.con.rollback()
             raise
     return decorated_f
 
@@ -171,12 +174,12 @@ class FileTotals(object):
             if state_for_db is TRANSLATED:
                 record['translated'] = 1
                 record['translatedsourcewords'] = sourcewords
-                record['translatedtargetwords'] = targetwords                
+                record['translatedtargetwords'] = targetwords
             elif state_for_db is FUZZY:
                 record['fuzzy'] = 1
                 record['fuzzysourcewords'] = sourcewords
         return record
-        
+
     new_record = classmethod(new_record)
 
     def _compute_derived_values(cls, record):
@@ -250,11 +253,12 @@ class StatsCache(object):
     """The current cursor"""
 
     def __new__(cls, statsfile=None):
+        current_thread = thread.get_ident()
         def make_database(statsfile):
             def connect(cache):
                 cache.con = dbapi2.connect(statsfile)
                 cache.cur = cache.con.cursor()
-            
+
             def clear_old_data(cache):
                 try:
                     cache.cur.execute("""SELECT toolkitbuild FROM files""")
@@ -262,6 +266,7 @@ class StatsCache(object):
                     # If the database is empty, we have no idea whether its layout
                     # is correct, so we might as well delete it.
                     if val is None or val[0] < toolkitversion.build:
+                        cache.con.close()
                         del cache
                         os.unlink(statsfile)
                         return True
@@ -269,7 +274,7 @@ class StatsCache(object):
                 except dbapi2.OperationalError:
                     return False
             
-            cache = cls._caches[statsfile] = object.__new__(cls)
+            cache = cls._caches.setdefault(current_thread, {})[statsfile] = object.__new__(cls)
             connect(cache)
             if clear_old_data(cache):
                 connect(cache)
@@ -291,13 +296,14 @@ class StatsCache(object):
         else:
             statsfile = os.path.realpath(statsfile)
         # First see if a cache for this file already exists:
-        if statsfile in cls._caches:
-            return cls._caches[statsfile]
-        # No existing cache. Let's build a new one and keep a copy        
+        if current_thread in cls._caches and statsfile in cls._caches[current_thread]:
+            return cls._caches[current_thread][statsfile]
+        # No existing cache. Let's build a new one and keep a copy
         return make_database(statsfile)
 
+    @transaction
     def create(self):
-        """Create all tables and indexes."""        
+        """Create all tables and indexes."""
         self.file_totals = FileTotals(self.cur)
 
         self.cur.execute("""CREATE TABLE IF NOT EXISTS files(
@@ -341,17 +347,18 @@ class StatsCache(object):
 
         self.cur.execute("""CREATE INDEX IF NOT EXISTS uniterrorindex
             ON uniterrors(fileid, configid);""")
-    create = transaction(create)
 
+    @transaction
     def _getfileid(self, filename, check_mod_info=True, store=None):
-        """Attempt to find the fileid of the given file, if it hasn't been
-           updated since the last record update.
+        """return fileid representing the given file in the statscache.
 
-           None is returned if either the file's record is not found, or if it is
-           not up to date.
+        if file not in cache or has been updated since last record
+        update, recalculate stats.
 
-           @param filename: the filename to retrieve the id for
-           @rtype: String or None
+        optional argument store can be used to avoid unnessecary
+        reparsing of already loaded translation files.
+
+        store can be a TranslationFile object or a callback that returns one.
         """
         realpath = os.path.realpath(filename)
         self.cur.execute("""SELECT fileid, st_mtime, st_size FROM files
@@ -368,11 +375,15 @@ class StatsCache(object):
                 return fileid
             if (filerow[1], filerow[2]) == mod_info:
                 return fileid
-        # We can only ignore the mod_info if the row already exists:
-        assert check_mod_info
-        store = store or factory.getobject(realpath)
-        return self._cachestore(store, realpath, mod_info)
 
+        # file wasn't in db at all, lets recache it
+        if callable(store):
+            store = store()
+        else:
+            store = store or factory.getobject(realpath)
+
+        return self._cachestore(store, realpath, mod_info)
+    
     def _getstoredcheckerconfig(self, checker):
         """See if this checker configuration has been used before."""
         config = str(checker.config.__dict__)
@@ -384,6 +395,7 @@ class StatsCache(object):
         else:
             return configrow[0]
 
+    @transaction
     def _cacheunitstats(self, units, fileid, unitindex=None, file_totals_record=FileTotals.new_record()):
         """Cache the statistics for the supplied unit(s)."""
         unitvalues = []
@@ -408,6 +420,7 @@ class StatsCache(object):
             return state_strings[statefordb(units[0])]
         return ""
 
+    @transaction
     def _cachestore(self, store, realpath, mod_info):
         """Calculates and caches the statistics of the given store
         unconditionally."""
@@ -422,12 +435,12 @@ class StatsCache(object):
         self._cacheunitstats(store.units, fileid)
         return fileid
 
-    def filetotals(self, filename):
+    def filetotals(self, filename, store=None):
         """Retrieves the statistics for the given file if possible, otherwise
         delegates to cachestore()."""
-        return self.file_totals[self._getfileid(filename)]
-    filetotals = transaction(filetotals)
+        return self.file_totals[self._getfileid(filename, store=store)]
 
+    @transaction
     def _cacheunitschecks(self, units, fileid, configid, checker, unitindex=None):
         """Helper method for cachestorechecks() and recacheunit()"""
         # We always want to store one dummy error to know that we have actually
@@ -460,6 +473,7 @@ class StatsCache(object):
             unitvalues)
         return errornames
 
+    @transaction
     def _cachestorechecks(self, fileid, store, checker, configid):
         """Calculates and caches the error statistics of the given store
         unconditionally."""
@@ -481,12 +495,13 @@ class StatsCache(object):
             return result
         else:
             print >> sys.stderr, """WARNING: Database in inconsistent state. 
-            fileid %d and unitid %d have no entries in the table units.""" % (fileid, unitid)
+            fileid %d and unitid %s have no entries in the table units.""" % (fileid, unitid)
             # If values.fetchone() is None, then we return an empty list,
             # to make FileTotals.new_record(*self.get_unit_stats(fileid, unitid))
             # do the right thing.
             return []
 
+    @transaction
     def recacheunit(self, filename, checker, unit):
         """Recalculate all information for a specific unit. This is necessary
         for updating all statistics when a translation of a unit took place,
@@ -495,7 +510,7 @@ class StatsCache(object):
         This method assumes that everything was up to date before (file totals,
         checks, checker config, etc."""
         fileid = self._getfileid(filename, check_mod_info=False)
-        configid = self._getstoredcheckerconfig(checker)
+        configid = self._get_config_id(fileid, checker)
         unitid = unit.getid()
         # get the unit index
         totals_without_unit = self.file_totals[fileid] - \
@@ -513,7 +528,6 @@ class StatsCache(object):
             checker.setsuggestionstore(factory.getobject(suggestion_filename(filename), ignore=suggestion_extension()))
         state.extend(self._cacheunitschecks([unit], fileid, configid, checker, unitindex))
         return state
-    recacheunit = transaction(recacheunit)
 
     def _checkerrors(self, filename, fileid, configid, checker, store):
         def geterrors():
@@ -527,10 +541,14 @@ class StatsCache(object):
         first, cur = geterrors()
         if first is not None:
             return first, cur
-        
+
         # This could happen if we haven't done the checks before, or the
         # file changed, or we are using a different configuration
-        store = store or factory.getobject(filename)
+        if callable(store):
+            store = store()
+        else:
+            store = store or factory.getobject(filename)
+
         if os.path.exists(suggestion_filename(filename)):
             checker.setsuggestionstore(factory.getobject(suggestion_filename(filename), ignore=suggestion_extension()))
         self._cachestorechecks(fileid, store, checker, configid)
@@ -543,6 +561,7 @@ class StatsCache(object):
         result.extend(cur.fetchall())
         return result
 
+    @transaction
     def _get_config_id(self, fileid, checker):
         configid = self._getstoredcheckerconfig(checker)
         if configid:
@@ -569,7 +588,6 @@ class StatsCache(object):
             errors[checkkey].append(value[1])   #value[1] is the unitindex
 
         return errors
-    filechecks = transaction(filechecks)
 
     def file_fails_test(self, filename, checker, name):
         fileid = self._getfileid(filename)
@@ -581,13 +599,11 @@ class StatsCache(object):
             FROM uniterrors 
             WHERE fileid=? and configid=? and name=?;""", (fileid, configid, name))
         return self.cur.fetchone() is not None
-    file_fails_test = transaction(file_fails_test)
-        
-    def filestats(self, filename, checker, store=None):
-        """Return a dictionary of property names mapping sets of unit
-        indices with those properties."""
+
+    def filestatestats(self, filename, store=None):
+        """Return a dictionary of unit stats mapping sets of unit
+        indices with those states"""
         stats = emptyfilestats()
-        stats.update(self.filechecks(filename, checker, store))
         fileid = self._getfileid(filename, store=store)
 
         self.cur.execute("""SELECT
@@ -595,14 +611,21 @@ class StatsCache(object):
             unitindex
             FROM units WHERE fileid=?
             ORDER BY unitindex;""", (fileid,))
-
         values = self.cur.fetchall()
+
         for value in values:
             stats[state_strings[value[0]]].append(value[1])
             stats["total"].append(value[1])
 
         return stats
-    filestats = transaction(filestats)
+
+    def filestats(self, filename, checker, store=None):
+        """Return a dictionary of property names mapping sets of unit
+        indices with those properties."""
+        stats = emptyfilestats()
+        stats.update(self.filechecks(filename, checker, store))
+        stats.update(self.filestatestats(filename, store))
+        return stats
 
     def unitstats(self, filename, _lang=None, store=None):
         # For now, lang and store are unused. lang will allow the user to
@@ -629,4 +652,3 @@ class StatsCache(object):
             stats["targetwordcount"].append(targetcount)
 
         return stats
-    unitstats = transaction(unitstats)
