@@ -1,9 +1,28 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+#
+# Copyright 2008-2009 Zuza Software Foundation
+#
+# This file is part of the Translate Toolkit.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 """tests for storage base classes"""
 
+from translate.misc.multistring import multistring
 from translate.storage import base
+from translate.storage.placeables import general, parse as rich_parse
 from py import test
 import os
 import warnings
@@ -127,6 +146,44 @@ class TestTranslationUnit:
         actual_notes = unit.getnotes()
         assert actual_notes == expected_notes
 
+    def test_rich_get(self):
+        """Basic test for converting from multistrings to StringElem trees."""
+        target_mstr = multistring([u'tėst', u'<b>string</b>'])
+        unit = self.UnitClass(multistring([u'a', u'b']))
+        unit.rich_parsers = general.parsers
+        unit.target = target_mstr
+        elems = unit.rich_target
+
+        if unit.hasplural():
+            assert len(elems) == 2
+            assert len(elems[0].sub) == 1
+            assert len(elems[1].sub) == 3
+
+            assert unicode(elems[0]) == target_mstr.strings[0]
+            assert unicode(elems[1]) == target_mstr.strings[1]
+
+            assert unicode(elems[1].sub[0]) == u'<b>'
+            assert unicode(elems[1].sub[1]) == u'string'
+            assert unicode(elems[1].sub[2]) == u'</b>'
+        else:
+            assert len(elems[0].sub) == 1
+            assert unicode(elems[0]) == target_mstr.strings[0]
+
+    def test_rich_set(self):
+        """Basic test for converting from multistrings to StringElem trees."""
+        elems = [
+            rich_parse(u'Tëst <x>string</x>', general.parsers),
+            rich_parse(u'Another test string.', general.parsers),
+        ]
+        unit = self.UnitClass(multistring([u'a', u'b']))
+        unit.rich_target = elems
+
+        if unit.hasplural():
+            assert unit.target.strings[0] == u'Tëst <x>string</x>'
+            assert unit.target.strings[1] == u'Another test string.'
+        else:
+            assert unit.target == u'Tëst <x>string</x>'
+
 class TestTranslationStore(object):
     """Tests a TranslationStore.
     Derived classes can reuse these tests by pointing StoreClass to a derived Store"""
