@@ -2,10 +2,10 @@
 # -*- coding: utf-8 -*-
 
 from translate.storage import po
+from translate.storage import pypo
 from translate.storage import test_base
 from translate.misc import wStringIO
 from translate.misc.multistring import multistring
-from translate.storage.placeables import X, G
 from py.test import raises
 
 def test_roundtrip_quoting():
@@ -15,8 +15,8 @@ def test_roundtrip_quoting():
                 '\n', '\t', '\r', 
                 '\\n', '\\t', '\\r', '\\"', '\r\n', '\\r\\n', '\\']
     for special in specials:
-        quoted_special = po.quoteforpo(special)
-        unquoted_special = po.unquotefrompo(quoted_special)
+        quoted_special = pypo.quoteforpo(special)
+        unquoted_special = pypo.unquotefrompo(quoted_special)
         print "special: %r\nquoted: %r\nunquoted: %r\n" % (special, quoted_special, unquoted_special)
         assert special == unquoted_special
 
@@ -90,17 +90,15 @@ class TestPOUnit(test_base.TestTranslationUnit):
         # plain text, no plural test
         unit = self.UnitClass("Tree")
         unit.target = "ki"
-        assert unit.target.strings == ["ki"]
-        assert unit.source.strings == ["Tree"]
         assert unit.hasplural() == False
-        
+
         # plural test with multistring
         unit.setsource(["Tree", "Trees"])
         assert unit.source.strings == ["Tree", "Trees"]
         assert unit.hasplural()
         unit.target = multistring(["ki", "ni ki"])
         assert unit.target.strings == ["ki", "ni ki"]
-        
+
         # test of msgid with no plural and msgstr with plural
         unit = self.UnitClass("Tree")
         assert raises(ValueError, unit.settarget, [u"ki", u"ni ki"])
@@ -166,6 +164,18 @@ class TestPOFile(test_base.TestTranslationStore):
         print oldunit
         return str(oldunit)
 
+    def test_context_only(self):
+        """Checks that an empty msgid with msgctxt is handled correctly."""
+        posource = '''msgctxt "CONTEXT"
+msgid ""
+msgstr ""
+'''
+        pofile = self.poparse(posource)
+        assert pofile.units[0].istranslatable()
+        assert not pofile.units[0].isheader()
+        # we were not generating output for thse at some stage
+        assert str(pofile)
+
     def test_simpleentry(self):
         """checks that a simple po entry is parsed correctly"""
         posource = '#: test.c:100 test.c:101\nmsgid "test"\nmsgstr "rest"\n'
@@ -378,6 +388,7 @@ msgstr "een"
         unit = pofile.units[1]
         assert unit.isobsolete()
 
+        print str(pofile)
         assert str(pofile) == posource
         unit.resurrect()
         assert unit.hasplural()
@@ -437,8 +448,10 @@ msgstr[1] "Koeie"
         print pofile
         unit = pofile.units[0]
         assert not unit.isobsolete()
+        assert unit.istranslatable()
         unit.makeobsolete()
         assert unit.isobsolete()
+        assert not unit.istranslatable()
         print pofile
         assert str(unit) == poexpected
 
@@ -649,4 +662,20 @@ msgstr "b"
         pofile = self.poparse(posource)
         for line in pofile.units[0].getnotes():
             assert isinstance(line, unicode)
-        
+
+    def test_final_slash(self):
+        """Test that \ as last character is correcly interpreted (bug 960)."""
+        posource = r'''
+msgid ""
+msgstr ""
+"Content-Type: text/plain; charset=utf-8\n"
+
+#: System-Support,Project>>decideAboutCreatingBlank:
+msgid "I cannot locate the project\\"
+msgstr "プロジェクトが見つかりませんでした"
+'''
+        pofile1 = self.poparse(posource)
+        print pofile1.units[1].source
+        pofile2 = self.poparse(str(pofile1))
+        print str(pofile2)
+        assert str(pofile1) == str(pofile2)
