@@ -1,24 +1,22 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright 2008 Zuza Software Foundation
-# 
-# This file is part of translate.
+# Copyright 2008-2009 Zuza Software Foundation
 #
-# translate is free software; you can redistribute it and/or modify
+# This file is part of the Translate Toolkit.
+#
+# This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
-# translate is distributed in the hope that it will be useful,
+#
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-#
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 """Module for handling Qt linguist (.ts) files.
 
@@ -27,6 +25,7 @@ format. While converters haven't been updated to use this module, we retain
 both.
 
 U{TS file format 4.3<http://doc.trolltech.com/4.3/linguist-ts-file-format.html>}, 
+U{4.5<http://doc.trolltech.com/4.5/linguist-ts-file-format.html>},
 U{Example<http://svn.ez.no/svn/ezcomponents/trunk/Translation/docs/linguist-format.txt>}, 
 U{Plurals forms<http://www.koders.com/cpp/fidE7B7E83C54B9036EB7FA0F27BC56BCCFC4B9DF34.aspx#L200>}
 
@@ -34,7 +33,8 @@ U{Specification of the valid variable entries <http://doc.trolltech.com/4.3/qstr
 U{2 <http://doc.trolltech.com/4.3/qstring.html#arg-2>}
 """
 
-from translate.storage import lisa
+from translate.storage import base, lisa
+from translate.storage.placeables import general, StringElem
 from translate.misc.multistring import multistring
 from translate.lang import data
 from lxml import etree
@@ -67,6 +67,7 @@ class tsunit(lisa.LISAunit):
     languageNode = "source"
     textNode = ""
     namespace = ''
+    rich_parsers = general.parsers
 
     def createlanguageNode(self, lang, text, purpose):
         """Returns an xml Element setup with given parameters."""
@@ -101,6 +102,7 @@ class tsunit(lisa.LISAunit):
         else:
             return data.forceunicode(sourcenode.text)
     source = property(getsource, lisa.LISAunit.setsource)
+    rich_source = property(base.TranslationUnit._get_rich_source, base.TranslationUnit._set_rich_source)
 
     def settarget(self, text):
         # This is a fairly destructive implementation. Don't assume that this
@@ -121,7 +123,8 @@ class tsunit(lisa.LISAunit):
         targetnode.clear()
         if type:
             targetnode.set("type", type)
-        if self.hasplural():
+        if self.hasplural() or len(strings) > 1:
+            self.xmlelement.set("numerus", "yes")
             for string in strings:
                 numerus = etree.SubElement(targetnode, self.namespaced("numerusform"))
                 numerus.text = data.forceunicode(string) or u""
@@ -139,6 +142,7 @@ class tsunit(lisa.LISAunit):
         else:
             return data.forceunicode(targetnode.text) or u""
     target = property(gettarget, settarget)
+    rich_target = property(base.TranslationUnit._get_rich_target, base.TranslationUnit._set_rich_target)
 
     def hasplural(self):
         return self.xmlelement.get("numerus") == "yes"
@@ -168,7 +172,10 @@ class tsunit(lisa.LISAunit):
 
     def _gettype(self):
         """Returns the type of this translation."""
-        return self._gettargetnode().get("type")
+        targetnode = self._gettargetnode()
+        if targetnode is not None:
+            return targetnode.get("type")
+        return None
 
     def _settype(self, value=None):
         """Set the type of this translation."""
@@ -201,6 +208,14 @@ class tsunit(lisa.LISAunit):
         else:
             return self.source
 
+    def istranslatable(self):
+        # Found a file in the wild with no context and an empty source. This
+        # served as a header, so let's classify this as not translatable.
+        # http://bibletime.svn.sourceforge.net/viewvc/bibletime/trunk/bibletime/i18n/messages/bibletime_ui.ts
+        # Furthermore, let's decide to handle obsolete units as untranslatable
+        # like we do with PO.
+        return bool(self.getid()) and not self.isobsolete()
+
     def getcontext(self):
         return self.xmlelement.getparent().find("name").text
 
