@@ -1,34 +1,36 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright 2005-2007 Zuza Software Foundation
-# 
-# This file is part of translate.
+# Copyright 2005-2009 Zuza Software Foundation
 #
-# translate is free software; you can redistribute it and/or modify
+# This file is part of the Translate Toolkit.
+#
+# This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
-# translate is distributed in the hope that it will be useful,
+#
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with translate; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-#
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 """Module for handling XLIFF files for translation.
 
 The official recommendation is to use the extention .xlf for XLIFF files.
 """
 
-from translate.storage import base
-from translate.storage import lisa
 from lxml import etree
 
+from translate.misc.multistring import multistring
+from translate.misc.xml_helpers import *
+from translate.storage import base, lisa
+from translate.storage.lisa import getXMLspace
+from translate.storage.placeables.lisa import xml_to_strelem, strelem_to_xml
+
 # TODO: handle translation types
 
 class xliffunit(lisa.LISAunit):
@@ -39,8 +41,17 @@ class xliffunit(lisa.LISAunit):
     textNode = ""
     namespace = 'urn:oasis:names:tc:xliff:document:1.1'
 
+    _default_xml_space = "default"
+
     #TODO: id and all the trans-unit level stuff
 
+    def __init__(self, source, empty=False, **kwargs):
+        """Override the constructor to set xml:space="preserve"."""
+        if empty:
+            return
+        super(xliffunit, self).__init__(source, empty, **kwargs)
+        lisa.setXMLspace(self.xmlelement, "preserve")
+
     def createlanguageNode(self, lang, text, purpose):
         """Returns an xml Element setup with given parameters."""
 
@@ -55,23 +66,70 @@ class xliffunit(lisa.LISAunit):
 #        self.createPHnodes(langset, text)
         langset.text = text
         return langset
-    
+
     def getlanguageNodes(self):
         """We override this to get source and target nodes."""
-        sources = list(self.xmlelement.iterdescendants(self.namespaced(self.languageNode)))
-        targets = list(self.xmlelement.iterdescendants(self.namespaced('target')))
-        sourcesl = len(sources)
-        targetsl = len(targets)
+        source = None
+        target = None
         nodes = []
-        for pair in zip(sources, targets):
-            nodes.extend(list(pair))
-        if sourcesl > targetsl:
-            nodes.extend(sources[- (sourcesl - targetsl):])
+        try:
+            source = self.xmlelement.iterchildren(self.namespaced(self.languageNode)).next()
+            target = self.xmlelement.iterchildren(self.namespaced('target')).next()
+            nodes = [source, target]
+        except StopIteration:
+            if source is not None:
+                nodes.append(source)
+            if not target is None:
+                nodes.append(target)
         return nodes
 
+    def set_rich_source(self, value, sourcelang='en'):
+        sourcelanguageNode = self.get_source_dom()
+        if sourcelanguageNode is None:
+            sourcelanguageNode = self.createlanguageNode(sourcelang, u'', "source")
+            self.set_source_dom(sourcelanguageNode)
+
+        # Clear sourcelanguageNode first
+        for i in range(len(sourcelanguageNode)):
+            del sourcelanguageNode[0]
+        sourcelanguageNode.text = None
+
+        strelem_to_xml(sourcelanguageNode, value[0])
+
+    def get_rich_source(self):
+        #rsrc = xml_to_strelem(self.source_dom)
+        #logging.debug('rich source: %s' % (repr(rsrc)))
+        #from dubulib.debug.misc import print_stack_funcs
+        #print_stack_funcs()
+        return [xml_to_strelem(self.source_dom, getXMLspace(self.xmlelement, self._default_xml_space))]
+    rich_source = property(get_rich_source, set_rich_source)
+
+    def set_rich_target(self, value, lang='xx', append=False):
+        if value is None:
+            self.set_target_dom(self.createlanguageNode(lang, u'', "target"))
+            return
+
+        languageNode = self.get_target_dom()
+        if languageNode is None:
+            languageNode = self.createlanguageNode(lang, u'', "target")
+            self.set_target_dom(languageNode, append)
+
+        # Clear languageNode first
+        for i in range(len(languageNode)):
+            del languageNode[0]
+        languageNode.text = None
+
+        strelem_to_xml(languageNode, value[0])
+
+    def get_rich_target(self, lang=None):
+        """retrieves the "target" text (second entry), or the entry in the
+        specified language, if it exists"""
+        return [xml_to_strelem(self.get_target_dom(lang), getXMLspace(self.xmlelement, self._default_xml_space))]
+    rich_target = property(get_rich_target, set_rich_target)
+
     def addalttrans(self, txt, origin=None, lang=None, sourcetxt=None, matchquality=None):
         """Adds an alt-trans tag and alt-trans components to the unit.
-        
+
         @type txt: String
         @param txt: Alternative translation of the source text.
         """
@@ -81,13 +139,14 @@ class xliffunit(lisa.LISAunit):
         if isinstance(txt, str):
             txt = txt.decode("utf-8")
         alttrans = etree.SubElement(self.xmlelement, self.namespaced("alt-trans"))
-        alttarget = etree.SubElement(alttrans, self.namespaced("target"))
-        alttarget.text = txt
+        lisa.setXMLspace(alttrans, "preserve")
         if sourcetxt:
             if isinstance(sourcetxt, str):
                 sourcetxt = sourcetxt.decode("utf-8")
             altsource = etree.SubElement(alttrans, self.namespaced("source"))
             altsource.text = sourcetxt
+        alttarget = etree.SubElement(alttrans, self.namespaced("target"))
+        alttarget.text = txt
         if matchquality:
             alttrans.set("match-quality", matchquality)
         if origin:
@@ -108,13 +167,13 @@ class xliffunit(lisa.LISAunit):
                 # the source tag is optional
                 sourcenode = node.iterdescendants(self.namespaced("source"))
                 try:
-                    newunit.source = lisa.getText(sourcenode.next())
+                    newunit.source = lisa.getText(sourcenode.next(), getXMLspace(node, self._default_xml_space))
                 except StopIteration:
                     pass
 
                 # must have one or more targets
                 targetnode = node.iterdescendants(self.namespaced("target"))
-                newunit.target = lisa.getText(targetnode.next())
+                newunit.target = lisa.getText(targetnode.next(), getXMLspace(node, self._default_xml_space))
                 #TODO: support multiple targets better
                 #TODO: support notes in alt-trans
                 newunit.xmlelement = node
@@ -128,10 +187,14 @@ class xliffunit(lisa.LISAunit):
 
     def addnote(self, text, origin=None):
         """Add a note specifically in a "note" tag"""
+        if text:
+            text = text.strip()
+        if not text:
+            return
         if isinstance(text, str):
             text = text.decode("utf-8")
         note = etree.SubElement(self.xmlelement, self.namespaced("note"))
-        note.text = text.strip()
+        note.text = text
         if origin:
             note.set("from", origin)
 
@@ -141,13 +204,13 @@ class xliffunit(lisa.LISAunit):
         # TODO: consider using xpath to construct initial_list directly
         # or to simply get the correct text from the outset (just remember to
         # check for duplication.
-        initial_list = [lisa.getText(note) for note in notenodes if self.correctorigin(note, origin)]
+        initial_list = [lisa.getText(note, getXMLspace(self.xmlelement, self._default_xml_space)) for note in notenodes if self.correctorigin(note, origin)]
 
         # Remove duplicate entries from list:
         dictset = {}
         notelist = [dictset.setdefault(note, note) for note in initial_list if note not in dictset]
 
-        return notelist 
+        return notelist
 
     def getnotes(self, origin=None):
         return '\n'.join(self.getnotelist(origin=origin))
@@ -209,7 +272,7 @@ class xliffunit(lisa.LISAunit):
 #                (targetnode.get("state-qualifier") == "fuzzy-match" or \
 #                targetnode.get("state") == "needs-review-translation")
         return not self.isapproved()
-                
+
     def markfuzzy(self, value=True):
         if value:
             self.markapproved(False)
@@ -290,15 +353,16 @@ class xliffunit(lisa.LISAunit):
         """Returns the contexts in the context groups with the specified name"""
         groups = []
         grouptags = self.xmlelement.iterdescendants(self.namespaced("context-group"))
+        #TODO: conbine name in query
         for group in grouptags:
             if group.get("name") == name:
                 contexts = group.iterdescendants(self.namespaced("context"))
                 pairs = []
                 for context in contexts:
-                    pairs.append((context.get("context-type"), lisa.getText(context)))
+                    pairs.append((context.get("context-type"), lisa.getText(context, getXMLspace(self.xmlelement, self._default_xml_space))))
                 groups.append(pairs) #not extend
         return groups
-        
+
     def getrestype(self):
         """returns the restype attribute in the trans-unit tag"""
         return self.xmlelement.get("restype")
@@ -326,10 +390,29 @@ class xliffunit(lisa.LISAunit):
         else:
             return False
 
+    def multistring_to_rich(self, mstr):
+        """Override L{TranslationUnit.multistring_to_rich} which is used by the
+            C{rich_source} and C{rich_target} properties."""
+        strings = mstr
+        if isinstance(mstr, multistring):
+            strings = mstr.strings
+        elif isinstance(mstr, basestring):
+            strings = [mstr]
+
+        return [xml_to_strelem(s) for s in strings]
+    multistring_to_rich = classmethod(multistring_to_rich)
+
+    def rich_to_multistring(self, elem_list):
+        """Override L{TranslationUnit.rich_to_multistring} which is used by the
+            C{rich_source} and C{rich_target} properties."""
+        return multistring([unicode(elem) for elem in elem_list])
+    rich_to_multistring = classmethod(rich_to_multistring)
+
+
 class xlifffile(lisa.LISAfile):
     """Class representing a XLIFF file store."""
     UnitClass = xliffunit
-    Name = _("XLIFF file")
+    Name = _("XLIFF Translation File")
     Mimetypes  = ["application/x-xliff", "application/x-xliff+xml"]
     Extensions = ["xlf", "xliff"]
     rootNode = "xliff"
@@ -342,13 +425,23 @@ class xlifffile(lisa.LISAfile):
 </file>
 </xliff>'''
     namespace = 'urn:oasis:names:tc:xliff:document:1.1'
+    suggestions_in_format = True
+    """xliff units have alttrans tags which can be used to store suggestions"""
 
     def __init__(self, *args, **kwargs):
+        self._filename = None
         lisa.LISAfile.__init__(self, *args, **kwargs)
-        self._filename = "NoName"
         self._messagenum = 0
 
-        # Allow the inputfile to override defaults for source and target language.
+
+    def initbody(self):
+        self.namespace = self.document.getroot().nsmap.get(None, None)
+
+        if self._filename:
+            self.body = self.getcontextnode(self._filename)
+        else:
+            self.body = self.document.getroot()
+
         filenode = self.document.getroot().iterchildren(self.namespaced('file')).next()
         sourcelanguage = filenode.get('source-language')
         if sourcelanguage:
@@ -385,6 +478,10 @@ class xlifffile(lisa.LISAfile):
         """returns the name of the given file"""
         return filenode.get("original")
 
+    def setfilename(self, filenode, filename):
+        """set the name of the given file"""
+        return filenode.set("original", filename)
+
     def getfilenames(self):
         """returns all filenames in this XLIFF file"""
         filenodes = self.document.getroot().iterchildren(self.namespaced("file"))
@@ -473,7 +570,6 @@ class xlifffile(lisa.LISAfile):
         unit = super(xlifffile, self).addsourceunit(source)
         self._messagenum += 1
         unit.setid("%d" % self._messagenum)
-        lisa.setXMLspace(unit.xmlelement, "preserve")
         return unit
 
     def switchfile(self, filename, createifmissing=False):
@@ -496,7 +592,7 @@ class xlifffile(lisa.LISAfile):
         #instead of globally in the whole XLIFF file, but using len(self.units)
         #will be much faster
         return True
-    
+
     def creategroup(self, filename="NoName", createifmissing=False, restype=None):
         """adds a group tag into the specified file"""
         if self._filename != filename:
@@ -506,7 +602,7 @@ class xlifffile(lisa.LISAfile):
         if restype:
             group.set("restype", restype)
         return group
-        
+
     def __str__(self):
         self.removedefaultfile()
         return super(xlifffile, self).__str__()
@@ -523,4 +619,3 @@ class xlifffile(lisa.LISAfile):
                 xliff = poxliff.PoXliffFile.parsestring(storestring)
         return xliff
     parsestring = classmethod(parsestring)
-
